/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Datasource.qpp Datasource class definition

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/QC_Datasource.h"
#include "qore/intern/QoreHashNodeIntern.h"
#include "qore/intern/QC_Serializable.h"

DLLLOCAL extern QoreClass* QC_ABSTRACTDATASOURCE;
DLLLOCAL extern QoreClass* QC_ABSTRACTSQLSTATEMENT;
DLLLOCAL extern QoreClass* QC_SQLSTATEMENT;

static const char* DSC_ERR = "DATASOURCE-CONSTRUCTOR-ERROR";

static int ds_set_options(ManagedDatasource* ds, const QoreHashNode* opts, ExceptionSink* xsink) {
    ConstHashIterator hi(opts);
    while (hi.next()) {
        // skip "min" and "max" options
        if (!strcmp(hi.getKey(), "min") || !strcmp(hi.getKey(), "max"))
            continue;

        if (ds->setOptionInit(hi.getKey(), hi.get(), xsink))
            return -1;
    }
    return 0;
}

static void ds_constructor_hash(const QoreHashNode* opts, QoreObject* self, Queue* queue, const QoreValue& arg,
        ExceptionSink* xsink) {
    ReferenceHolder<Queue> q(queue, xsink);

    const char* str = check_hash_key(opts, "type", DSC_ERR, xsink);
    if (*xsink) return;

    if (!str) {
        xsink->raiseException(DSC_ERR, "expecting a string value with the 'type' key giving the driver name");
        return;
    }

    DBIDriver* db_driver = DBI.find(str, xsink);
    if (!db_driver) {
        if (!*xsink)
            xsink->raiseException("DATASOURCE-UNSUPPORTED-DATABASE", "DBI driver '%s' cannot be loaded", str);
        return;
    }

    ReferenceHolder<ManagedDatasource> ds(new ManagedDatasource(db_driver), xsink);

    str = check_hash_key(opts, "user", DSC_ERR, xsink);
    if (*xsink) return;
    if (str) ds->setPendingUsername(str);

    str = check_hash_key(opts, "pass", DSC_ERR, xsink);
    if (*xsink) return;
    if (str) ds->setPendingPassword(str);

    str = check_hash_key(opts, "db", DSC_ERR, xsink);
    if (*xsink) return;
    if (str) ds->setPendingDBName(str);

    str = check_hash_key(opts, "charset", DSC_ERR, xsink);
    if (*xsink) return;
    if (str) ds->setPendingDBEncoding(str);

    str = check_hash_key(opts, "host", DSC_ERR, xsink);
    if (*xsink) return;
    if (str) ds->setPendingHostName(str);

    int port = (int)opts->getKeyValue("port").getAsBigInt();
    if (port) {
        if (port < 0) {
            xsink->raiseException(DSC_ERR, "port value must be zero (meaning use the default port) or positive "
                "(value given: %d)", port);
            return;
        }
        ds->setPendingPort(port);
    }

    {
        QoreValue p = opts->getKeyValue("options", xsink);
        if (!p.isNothing()) {
            if (p.getType() != NT_HASH) {
                xsink->raiseException(DSC_ERR, "'options' key is not type 'hash' but is type '%s'", p.getTypeName());
                return;
            }
            const QoreHashNode* opt_hash = p.get<const QoreHashNode>();
            if (ds_set_options(*ds, opt_hash, xsink))
                return;
        }
    }

    if (queue)
        ds->setEventQueue(q.release(), arg.refSelf(), xsink);

    self->setPrivate(CID_DATASOURCE, ds.release());
}

static void create_datasource_from_conf(const QoreString& conf, QoreObject& self, Queue* queue, QoreValue arg,
        ExceptionSink* xsink) {
    ReferenceHolder<Queue> holder(queue, xsink);

    qore_offset_t colon = conf.find(':');
    qore_offset_t atsign = conf.find('@');
    if (colon < 0 || (atsign < colon)) {
        // do not report the string here to avoid leaking password info in case present
        xsink->raiseException("DATASOURCE-PARSE-ERROR", "driver is missing from the datasource string");
        return;
    }

    ReferenceHolder<QoreHashNode> h(parseDatasource(conf.c_str(), xsink), xsink);
    if (!h) {
        assert(*xsink);
        return;
    }

    ds_constructor_hash(*h, &self, holder.release(), arg, xsink);
}

static QoreHashNode* Datasource_serializer(const QoreObject& self, const ManagedDatasource& ds,
        QoreSerializationContext& context, ExceptionSink* xsink) {
    SimpleRefHolder<QoreStringNode> str(const_cast<ManagedDatasource&>(ds).getConfigString(xsink));
    if (*xsink) {
        return nullptr;
    }

    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);
    rv->setKeyValue("conf", str.release(), xsink);
    return rv.release();
}

static void Datasource_deserializer(QoreObject& self, const QoreHashNode* sdata, QoreDeserializationContext& context,
        ExceptionSink* xsink) {
    if (sdata) {
        QoreValue val = sdata->getKeyValue("conf");
        if (val) {
            if (val.getType() != NT_STRING) {
                xsink->raiseException("DESERIALIZATION-ERROR", "Datasource serialization key 'conf' has type '%s'; "
                    "expecting 'string'", val.getTypeName());
                return;
            }
            create_datasource_from_conf(*val.get<QoreStringNode>(), self, nullptr, QoreValue(), xsink);
            return;
        }
    }

    xsink->raiseException("DESERIALIZATION-ERROR", "missing serialization data for the Datasource class");
}
/* Qore class Qore::SQL::Datasource */

qore_classid_t CID_DATASOURCE;
QoreClass* QC_DATASOURCE;

// nothing Datasource::beginTransaction(){}
static QoreValue Datasource_beginTransaction(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 957 "QC_Datasource.qpp"
    ds->beginTransaction(xsink);
    return QoreValue();
}

// nothing Datasource::clearEventQueue(){}
static QoreValue Datasource_clearEventQueue(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1457 "QC_Datasource.qpp"
    ds->setEventQueue(0, 0, xsink);
    return QoreValue();
}

// nothing Datasource::close(){}
static QoreValue Datasource_close(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 575 "QC_Datasource.qpp"
    ds->close(xsink);
    return QoreValue();
}

// nothing Datasource::commit(){}
static QoreValue Datasource_commit(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 590 "QC_Datasource.qpp"
    ds->commit(xsink);
    return QoreValue();
}

// Datasource::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, *softint port, *hash<auto> options, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_VsNsNsNsNsNsniC11_hash_auto_C5QueueVA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* driver = HARD_QORE_VALUE_STRING(args, 0);
    const QoreStringNode* user = get_param_value(args, 1).get<const QoreStringNode>();
    const QoreStringNode* pass = get_param_value(args, 2).get<const QoreStringNode>();
    const QoreStringNode* db = get_param_value(args, 3).get<const QoreStringNode>();
    const QoreStringNode* encoding = get_param_value(args, 4).get<const QoreStringNode>();
    const QoreStringNode* host = get_param_value(args, 5).get<const QoreStringNode>();
    int64 port = HARD_QORE_VALUE_INT(args, 6);
    const QoreHashNode* options = get_param_value(args, 7).get<const QoreHashNode>();
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 8, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 9);
# 389 "QC_Datasource.qpp"
ReferenceHolder<Queue> q(queue, xsink);

    if (!port && driver->find('@') != -1) {
        qore_offset_t colon = driver->find(':');
        qore_offset_t atsign = driver->find('@');
        if (colon < 0 || (atsign < colon)) {
            xsink->raiseException("DATASOURCE-PARSE-ERROR", "driver is missing from the datasource string");
            return;
        }

        ReferenceHolder<QoreHashNode> h(parseDatasource(driver->c_str(), xsink), xsink);
        if (!h) {
            assert(*xsink);
            return;
        }
        ds_constructor_hash(*h, self, q.release(), arg, xsink);
        return;
    }

    DBIDriver *db_driver = DBI.find(driver->c_str(), xsink);
    if (!db_driver) {
        if (!*xsink)
        xsink->raiseException("DATASOURCE-UNSUPPORTED-DATABASE", "DBI driver '%s' cannot be loaded", driver->c_str());
        return;
    }

    ReferenceHolder<ManagedDatasource> ds(new ManagedDatasource(db_driver), xsink);

    if (user && !user->empty())
        ds->setPendingUsername(user->c_str());

    if (pass && !pass->empty())
        ds->setPendingPassword(pass->c_str());

    if (db && !db->empty())
        ds->setPendingDBName(db->c_str());

    if (encoding && !encoding->empty())
        ds->setPendingDBEncoding(encoding->c_str());

    if (host && !host->empty())
        ds->setPendingHostName(host->c_str());

    if (port) {
        if (port < 0) {
            xsink->raiseException(DSC_ERR, "port value must be zero (meaning use the default port) or positive "
                "(value given: %d)", port);
            return;
        }
        ds->setPendingPort((int)port);
    }

    if (options)
        ds_set_options(*ds, options, xsink);

    if (queue)
        ds->setEventQueue(q.release(), arg.refSelf(), xsink);

    self->setPrivate(CID_DATASOURCE, ds.release());
}

// Datasource::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_VsC5QueueVA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* desc = HARD_QORE_VALUE_STRING(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 469 "QC_Datasource.qpp"
ReferenceHolder<Queue> q(queue, xsink);

    TempEncodingHelper tstr(desc, QCS_UTF8, xsink);
    if (*xsink) {
        return;
    }

    create_datasource_from_conf(**tstr, *self, q.release(), arg, xsink);
}

// Datasource::constructor(hash<auto> opts, *Qore::Thread::Queue queue, auto arg) {}
static void Datasource_constructor_C10hash_auto_C5QueueVA(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreHashNode* opts = HARD_QORE_VALUE_HASH(args, 0);
    HARD_QORE_VALUE_OBJ_OR_NOTHING_DATA(queue, Queue, args, 1, CID_QUEUE, xsink);
    if (*xsink)
        return;
    QoreValue arg = get_param_value(args, 2);
# 518 "QC_Datasource.qpp"
ds_constructor_hash(opts, self, queue, arg, xsink);
}

// Datasource::copy() {}
static void Datasource_copy(QoreObject* self, QoreObject* old, ManagedDatasource* ds, ExceptionSink* xsink) {
# 544 "QC_Datasource.qpp"
self->setPrivate(CID_DATASOURCE, ds->copy());
}

// bool Datasource::currentThreadInTransaction(){}
static QoreValue Datasource_currentThreadInTransaction(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1332 "QC_Datasource.qpp"
    return ds->currentThreadInTransaction();
}

// *hash<auto> Datasource::describe(string sql, ...){}
static QoreValue Datasource_describe_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 937 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->copyListFrom(1) : 0), xsink);
    return ds->describe(sql, *argl, xsink);
}

// Datasource::destructor() {}
static void Datasource_destructor(QoreObject* self, ManagedDatasource* ds, ExceptionSink* xsink) {
# 532 "QC_Datasource.qpp"
ds->destructor(xsink);
    ds->deref(xsink);
}

// auto Datasource::exec(string sql, ...){}
static QoreValue Datasource_exec_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 655 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->copyListFrom(1) : 0), xsink);
    return ds->exec(sql, *argl, xsink);
}

// auto Datasource::execRaw(string sql){}
static QoreValue Datasource_execRaw_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 714 "QC_Datasource.qpp"
    return ds->execRaw(sql, xsink);
}

// bool Datasource::getAutoCommit(){}
static QoreValue Datasource_getAutoCommit(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 629 "QC_Datasource.qpp"
    return ds->getAutoCommit();
}

// int Datasource::getCapabilities(){}
static QoreValue Datasource_getCapabilities(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 986 "QC_Datasource.qpp"
    return ds->getCapabilities();
}

// list<auto> Datasource::getCapabilityList(){}
static QoreValue Datasource_getCapabilityList(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1000 "QC_Datasource.qpp"
    return ds->getCapabilityList();
}

// auto Datasource::getClientVersion(){}
static QoreValue Datasource_getClientVersion(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1304 "QC_Datasource.qpp"
    return ds->getClientVersion(xsink);
}

// hash<auto> Datasource::getConfigHash(){}
static QoreValue Datasource_getConfigHash(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1414 "QC_Datasource.qpp"
    return ds->getConfigHash(xsink);
}

// string Datasource::getConfigString(){}
static QoreValue Datasource_getConfigString(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1431 "QC_Datasource.qpp"
    return ds->getConfigString(xsink);
}

// *string Datasource::getDBCharset(){}
static QoreValue Datasource_getDBCharset(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1150 "QC_Datasource.qpp"
    return ds->getPendingDBEncoding();
}

// *string Datasource::getDBEncoding(){}
static QoreValue Datasource_getDBEncoding(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1164 "QC_Datasource.qpp"
    return ds->getPendingDBEncoding();
}

// *string Datasource::getDBName(){}
static QoreValue Datasource_getDBName(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1136 "QC_Datasource.qpp"
    return ds->getPendingDBName();
}

// string Datasource::getDriverName(){}
static QoreValue Datasource_getDriverName(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1255 "QC_Datasource.qpp"
    return new QoreStringNode(ds->getDriver()->getName());
}

// string Datasource::getDriverRealName(){}
static QoreValue Datasource_getDriverRealName(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1290 "QC_Datasource.qpp"
    return ds->ManagedDatasource::getDriverRealName(xsink);
}

// *string Datasource::getHostName(){}
static QoreValue Datasource_getHostName(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1204 "QC_Datasource.qpp"
    return ds->getPendingHostName();
}

// string Datasource::getOSCharset(){}
static QoreValue Datasource_getOSCharset(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1178 "QC_Datasource.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
    return new QoreStringNode(enc ? enc->getCode() : "(unknown)");
}

// *string Datasource::getOSEncoding(){}
static QoreValue Datasource_getOSEncoding(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1191 "QC_Datasource.qpp"
    const QoreEncoding *enc = ds->getQoreEncoding();
    return enc ? new QoreStringNode(enc->getCode()) : 0;
}

// auto Datasource::getOption(string opt){}
static QoreValue Datasource_getOption_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* opt = HARD_QORE_VALUE_STRING(args, 0);
# 1397 "QC_Datasource.qpp"
    return ds->getOption(opt->c_str(), xsink);
}

// hash<auto> Datasource::getOptionHash(){}
static QoreValue Datasource_getOptionHash(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1367 "QC_Datasource.qpp"
    return ds->getOptionHash(xsink);
}

// *string Datasource::getPassword(){}
static QoreValue Datasource_getPassword(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1124 "QC_Datasource.qpp"
    return ds->getPendingPassword();
}

// *int Datasource::getPort(){}
static QoreValue Datasource_getPort(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1216 "QC_Datasource.qpp"
    int port = ds->getPort();
    return port ? QoreValue(port) : QoreValue();
}

// AbstractSQLStatement Datasource::getSQLStatement(){}
static QoreValue Datasource_getSQLStatement(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1471 "QC_Datasource.qpp"
    if (!ds->getDriver()->hasStatementAPI()) {
        xsink->raiseException("SQLSTATEMENT-ERROR", "DBI driver '%s' does not support the prepared statement API",
            ds->getDriver()->getName());
        return QoreValue();
    }

    return new QoreObject(QC_SQLSTATEMENT, getProgram(), new QoreSQLStatement(ds));
}

// auto Datasource::getServerVersion(){}
static QoreValue Datasource_getServerVersion(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1271 "QC_Datasource.qpp"
    return ds->getServerVersion(xsink);
}

// int Datasource::getTransactionLockTimeout(){}
static QoreValue Datasource_getTransactionLockTimeout(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1241 "QC_Datasource.qpp"
    return ds->getTransactionLockTimeout();
}

// *string Datasource::getUserName(){}
static QoreValue Datasource_getUserName(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1112 "QC_Datasource.qpp"
    return ds->getPendingUsername();
}

// bool Datasource::inTransaction(){}
static QoreValue Datasource_inTransaction(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1316 "QC_Datasource.qpp"
    return ds->isInTransaction();
}

// nothing Datasource::open(){}
static QoreValue Datasource_open(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 560 "QC_Datasource.qpp"
    ds->open(xsink);
    return QoreValue();
}

// nothing Datasource::reset(){}
static QoreValue Datasource_reset(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 972 "QC_Datasource.qpp"
    ds->reset(xsink);
    return QoreValue();
}

// nothing Datasource::rollback(){}
static QoreValue Datasource_rollback(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 605 "QC_Datasource.qpp"
    ds->rollback(xsink);
    return QoreValue();
}

// auto Datasource::select(string sql, ...){}
static QoreValue Datasource_select_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 752 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->copyListFrom(1) : 0), xsink);
    return ds->select(sql, *argl, xsink);
}

// *hash<auto> Datasource::selectRow(string sql, ...){}
static QoreValue Datasource_selectRow_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 779 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->copyListFrom(1) : 0), xsink);
    return ds->selectRow(sql, *argl, xsink);
}

// auto Datasource::selectRows(string sql, ...){}
static QoreValue Datasource_selectRows_VsVV(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
# 811 "QC_Datasource.qpp"
    ReferenceHolder<QoreListNode> argl((args->size() > 1 ? args->copyListFrom(1) : 0), xsink);
    return ds->selectRows(sql, *argl, xsink);
}

// nothing Datasource::setAutoCommit(bool ac = True){}
static QoreValue Datasource_setAutoCommit_Vb(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    bool ac = HARD_QORE_VALUE_BOOL(args, 0);
# 617 "QC_Datasource.qpp"
    ds->setAutoCommit(ac, xsink);
    return QoreValue();
}

// nothing Datasource::setDBCharset(string encoding){}
static QoreValue Datasource_setDBCharset_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 1058 "QC_Datasource.qpp"
    ds->setPendingDBEncoding(encoding->c_str());
    return QoreValue();
}

// nothing Datasource::setDBEncoding(string encoding){}
static QoreValue Datasource_setDBEncoding_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* encoding = HARD_QORE_VALUE_STRING(args, 0);
# 1072 "QC_Datasource.qpp"
    ds->setPendingDBEncoding(encoding->c_str());
    return QoreValue();
}

// nothing Datasource::setDBName(string db){}
static QoreValue Datasource_setDBName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* db = HARD_QORE_VALUE_STRING(args, 0);
# 1042 "QC_Datasource.qpp"
    ds->setPendingDBName(db->c_str());
    return QoreValue();
}

// nothing Datasource::setEventQueue(Qore::Thread::Queue queue, auto arg){}
static QoreValue Datasource_setEventQueue_C5QueueVA(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(queue, Queue, args, 0, CID_QUEUE, "Datasource::setEventQueue()", "Qore::Thread::Queue", xsink);
    if (*xsink)
        return 0;
    QoreValue arg = get_param_value(args, 1);
# 1444 "QC_Datasource.qpp"
    ReferenceHolder<Queue> q(queue, xsink);

    if (queue->getMax() != -1)
        return xsink->raiseException("QUEUE-ERROR", "the Queue object passed as a maximum size of %d entr%s, which could cause internal I/O to freeze in the Datasource object if used; use a Queue object with no maximum size for datasource event monitoring", queue->getMax(), queue->getMax() == 1 ? "y" : "ies");

    ds->setEventQueue(q.release(), arg.refSelf(), xsink);
    return QoreValue();
}

// nothing Datasource::setHostName(string host){}
static QoreValue Datasource_setHostName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* host = HARD_QORE_VALUE_STRING(args, 0);
# 1086 "QC_Datasource.qpp"
    ds->setPendingHostName(host->c_str());
    return QoreValue();
}

// nothing Datasource::setOption(string opt, auto val){}
static QoreValue Datasource_setOption_VsVA(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* opt = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue val = get_param_value(args, 1);
# 1383 "QC_Datasource.qpp"
    ds->setOption(opt->c_str(), val, xsink);
    return QoreValue();
}

// nothing Datasource::setPassword(string pass){}
static QoreValue Datasource_setPassword_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* pass = HARD_QORE_VALUE_STRING(args, 0);
# 1028 "QC_Datasource.qpp"
    ds->setPendingPassword(pass->c_str());
    return QoreValue();
}

// nothing Datasource::setPort(softint port = 0){}
static QoreValue Datasource_setPort_vi(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 port = HARD_QORE_VALUE_INT(args, 0);
# 1100 "QC_Datasource.qpp"
    ds->setPendingPort((int)port);
    return QoreValue();
}

// nothing Datasource::setTransactionLockTimeout(timeout timeout_ms = 0){}
static QoreValue Datasource_setTransactionLockTimeout_Vt(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 timeout_ms = HARD_QORE_VALUE_INT(args, 0);
# 1229 "QC_Datasource.qpp"
    ds->setTransactionLockTimeout((int)timeout_ms);
    return QoreValue();
}

// nothing Datasource::setUserName(string user){}
static QoreValue Datasource_setUserName_Vs(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* user = HARD_QORE_VALUE_STRING(args, 0);
# 1014 "QC_Datasource.qpp"
    ds->setPendingUsername(user->c_str());
    return QoreValue();
}

// int Datasource::transactionTid(){}
static QoreValue Datasource_transactionTid(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1351 "QC_Datasource.qpp"
    return ds->transactionTid();
}

// auto Datasource::vexec(string sql, *softlist<auto> vargs){}
static QoreValue Datasource_vexec_VsC15_softlist_auto_(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 684 "QC_Datasource.qpp"
    return ds->exec(sql, vargs, xsink);
}

// auto Datasource::vselect(string sql, *softlist<auto> vargs){}
static QoreValue Datasource_vselect_VsC15_softlist_auto_(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 847 "QC_Datasource.qpp"
    return ds->select(sql, vargs, xsink);
}

// *hash<auto> Datasource::vselectRow(string sql, *softlist<auto> vargs){}
static QoreValue Datasource_vselectRow_VsC15_softlist_auto_(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 874 "QC_Datasource.qpp"
    return ds->selectRow(sql, vargs, xsink);
}

// auto Datasource::vselectRows(string sql, *softlist<auto> vargs){}
static QoreValue Datasource_vselectRows_VsC15_softlist_auto_(QoreObject* self, ManagedDatasource* ds, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* sql = HARD_QORE_VALUE_STRING(args, 0);
    const QoreListNode* vargs = get_param_value(args, 1).get<const QoreListNode>();
# 907 "QC_Datasource.qpp"
    return ds->selectRows(sql, vargs, xsink);
}

DLLLOCAL void preinitDatasourceClass() {
    QC_DATASOURCE = new QoreClass("Datasource", "::Qore::SQL::Datasource", QDOM_DATABASE);
    CID_DATASOURCE = QC_DATASOURCE->getID();
    QC_DATASOURCE->setSystem();
}

DLLLOCAL QoreClass* initDatasourceClass(QoreNamespace& ns) {
    if (!QC_DATASOURCE)
        preinitDatasourceClass();

    // set serializer function
    QC_DATASOURCE->setSerializer(reinterpret_cast<q_serializer_t>(Datasource_serializer));

    // set deserializer function
    QC_DATASOURCE->setDeserializer(Datasource_deserializer);

    // set parent class
    assert(QC_ABSTRACTDATASOURCE);
    QC_DATASOURCE->addBuiltinVirtualBaseClass(QC_ABSTRACTDATASOURCE);

    // set parent class
    assert(QC_SERIALIZABLE);
    QC_DATASOURCE->addBuiltinVirtualBaseClass(QC_SERIALIZABLE);

    // nothing Datasource::beginTransaction(){}
    QC_DATASOURCE->addMethod("beginTransaction", (q_method_n_t)Datasource_beginTransaction, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::clearEventQueue(){}
    QC_DATASOURCE->addMethod("clearEventQueue", (q_method_n_t)Datasource_clearEventQueue, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::close(){}
    QC_DATASOURCE->addMethod("close", (q_method_n_t)Datasource_close, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::commit(){}
    QC_DATASOURCE->addMethod("commit", (q_method_n_t)Datasource_commit, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // Datasource::constructor(string driver, *string user, *string pass, *string db, *string encoding, *string host, *softint port, *hash<auto> options, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_VsNsNsNsNsNsniC11_hash_auto_C5QueueVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 10, stringTypeInfo, QORE_PARAM_NO_ARG, "driver", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "user", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "pass", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "db", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "host", softBigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "port", autoHashOrNothingTypeInfo, QORE_PARAM_NO_ARG, "options", QC_QUEUE->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // Datasource::constructor(string desc, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_VsC5QueueVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "desc", QC_QUEUE->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // Datasource::constructor(hash<auto> opts, *Qore::Thread::Queue queue, auto arg) {}
    QC_DATASOURCE->addConstructor(Datasource_constructor_C10hash_auto_C5QueueVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 3, autoHashTypeInfo, QORE_PARAM_NO_ARG, "opts", QC_QUEUE->getOrNothingTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // Datasource::copy() {}
    QC_DATASOURCE->setCopy((q_copy_t)Datasource_copy);

    // bool Datasource::currentThreadInTransaction(){}
    QC_DATASOURCE->addMethod("currentThreadInTransaction", (q_method_n_t)Datasource_currentThreadInTransaction, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // *hash<auto> Datasource::describe(string sql, ...){}
    QC_DATASOURCE->addMethod("describe", (q_method_n_t)Datasource_describe_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // Datasource::destructor() {}
    QC_DATASOURCE->setDestructor((q_destructor_t)Datasource_destructor);

    // auto Datasource::exec(string sql, ...){}
    QC_DATASOURCE->addMethod("exec", (q_method_n_t)Datasource_exec_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto Datasource::execRaw(string sql){}
    QC_DATASOURCE->addMethod("execRaw", (q_method_n_t)Datasource_execRaw_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // bool Datasource::getAutoCommit(){}
    QC_DATASOURCE->addMethod("getAutoCommit", (q_method_n_t)Datasource_getAutoCommit, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // int Datasource::getCapabilities(){}
    QC_DATASOURCE->addMethod("getCapabilities", (q_method_n_t)Datasource_getCapabilities, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // list<auto> Datasource::getCapabilityList(){}
    QC_DATASOURCE->addMethod("getCapabilityList", (q_method_n_t)Datasource_getCapabilityList, Public, QCF_CONSTANT, QDOM_DEFAULT, autoListTypeInfo);

    // auto Datasource::getClientVersion(){}
    QC_DATASOURCE->addMethod("getClientVersion", (q_method_n_t)Datasource_getClientVersion, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // hash<auto> Datasource::getConfigHash(){}
    QC_DATASOURCE->addMethod("getConfigHash", (q_method_n_t)Datasource_getConfigHash, Public, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);

    // string Datasource::getConfigString(){}
    QC_DATASOURCE->addMethod("getConfigString", (q_method_n_t)Datasource_getConfigString, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getDBCharset(){}
    QC_DATASOURCE->addMethod("getDBCharset", (q_method_n_t)Datasource_getDBCharset, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Datasource::getDBEncoding(){}
    QC_DATASOURCE->addMethod("getDBEncoding", (q_method_n_t)Datasource_getDBEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *string Datasource::getDBName(){}
    QC_DATASOURCE->addMethod("getDBName", (q_method_n_t)Datasource_getDBName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string Datasource::getDriverName(){}
    QC_DATASOURCE->addMethod("getDriverName", (q_method_n_t)Datasource_getDriverName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // string Datasource::getDriverRealName(){}
    QC_DATASOURCE->addMethod("getDriverRealName", (q_method_n_t)Datasource_getDriverRealName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getHostName(){}
    QC_DATASOURCE->addMethod("getHostName", (q_method_n_t)Datasource_getHostName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // string Datasource::getOSCharset(){}
    QC_DATASOURCE->addMethod("getOSCharset", (q_method_n_t)Datasource_getOSCharset, Public, QCF_CONSTANT, QDOM_DEFAULT, stringTypeInfo);

    // *string Datasource::getOSEncoding(){}
    QC_DATASOURCE->addMethod("getOSEncoding", (q_method_n_t)Datasource_getOSEncoding, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // auto Datasource::getOption(string opt){}
    QC_DATASOURCE->addMethod("getOption", (q_method_n_t)Datasource_getOption_Vs, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "opt");

    // hash<auto> Datasource::getOptionHash(){}
    QC_DATASOURCE->addMethod("getOptionHash", (q_method_n_t)Datasource_getOptionHash, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashTypeInfo);

    // *string Datasource::getPassword(){}
    QC_DATASOURCE->addMethod("getPassword", (q_method_n_t)Datasource_getPassword, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // *int Datasource::getPort(){}
    QC_DATASOURCE->addMethod("getPort", (q_method_n_t)Datasource_getPort, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntOrNothingTypeInfo);

    // AbstractSQLStatement Datasource::getSQLStatement(){}
    QC_DATASOURCE->addMethod("getSQLStatement", (q_method_n_t)Datasource_getSQLStatement, Public, QCF_NO_FLAGS, QDOM_DEFAULT, QC_ABSTRACTSQLSTATEMENT->getTypeInfo());

    // auto Datasource::getServerVersion(){}
    QC_DATASOURCE->addMethod("getServerVersion", (q_method_n_t)Datasource_getServerVersion, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo);

    // int Datasource::getTransactionLockTimeout(){}
    QC_DATASOURCE->addMethod("getTransactionLockTimeout", (q_method_n_t)Datasource_getTransactionLockTimeout, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // *string Datasource::getUserName(){}
    QC_DATASOURCE->addMethod("getUserName", (q_method_n_t)Datasource_getUserName, Public, QCF_CONSTANT, QDOM_DEFAULT, stringOrNothingTypeInfo);

    // bool Datasource::inTransaction(){}
    QC_DATASOURCE->addMethod("inTransaction", (q_method_n_t)Datasource_inTransaction, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // nothing Datasource::open(){}
    QC_DATASOURCE->addMethod("open", (q_method_n_t)Datasource_open, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::reset(){}
    QC_DATASOURCE->addMethod("reset", (q_method_n_t)Datasource_reset, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // nothing Datasource::rollback(){}
    QC_DATASOURCE->addMethod("rollback", (q_method_n_t)Datasource_rollback, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo);

    // auto Datasource::select(string sql, ...){}
    QC_DATASOURCE->addMethod("select", (q_method_n_t)Datasource_select_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // *hash<auto> Datasource::selectRow(string sql, ...){}
    QC_DATASOURCE->addMethod("selectRow", (q_method_n_t)Datasource_selectRow_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // auto Datasource::selectRows(string sql, ...){}
    QC_DATASOURCE->addMethod("selectRows", (q_method_n_t)Datasource_selectRows_VsVV, Public, QCF_USES_EXTRA_ARGS, QDOM_DEFAULT, autoTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "sql");

    // nothing Datasource::setAutoCommit(bool ac = True){}
    QC_DATASOURCE->addMethod("setAutoCommit", (q_method_n_t)Datasource_setAutoCommit_Vb, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, boolTypeInfo, QoreSimpleValue().assign(true), "ac");

    // nothing Datasource::setDBCharset(string encoding){}
    QC_DATASOURCE->addMethod("setDBCharset", (q_method_n_t)Datasource_setDBCharset_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing Datasource::setDBEncoding(string encoding){}
    QC_DATASOURCE->addMethod("setDBEncoding", (q_method_n_t)Datasource_setDBEncoding_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "encoding");

    // nothing Datasource::setDBName(string db){}
    QC_DATASOURCE->addMethod("setDBName", (q_method_n_t)Datasource_setDBName_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "db");

    // nothing Datasource::setEventQueue(Qore::Thread::Queue queue, auto arg){}
    QC_DATASOURCE->addMethod("setEventQueue", (q_method_n_t)Datasource_setEventQueue_C5QueueVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, QC_QUEUE->getTypeInfo(), QORE_PARAM_NO_ARG, "queue", autoTypeInfo, QORE_PARAM_NO_ARG, "arg");

    // nothing Datasource::setHostName(string host){}
    QC_DATASOURCE->addMethod("setHostName", (q_method_n_t)Datasource_setHostName_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "host");

    // nothing Datasource::setOption(string opt, auto val){}
    QC_DATASOURCE->addMethod("setOption", (q_method_n_t)Datasource_setOption_VsVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "opt", autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    // nothing Datasource::setPassword(string pass){}
    QC_DATASOURCE->addMethod("setPassword", (q_method_n_t)Datasource_setPassword_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "pass");

    // nothing Datasource::setPort(softint port = 0){}
    QC_DATASOURCE->addMethod("setPort", (q_method_n_t)Datasource_setPort_vi, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, softBigIntTypeInfo, QoreSimpleValue().assign((int64)0), "port");

    // nothing Datasource::setTransactionLockTimeout(timeout timeout_ms = 0){}
    QC_DATASOURCE->addMethod("setTransactionLockTimeout", (q_method_n_t)Datasource_setTransactionLockTimeout_Vt, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, timeoutTypeInfo, QoreSimpleValue().assign((int64)0), "timeout_ms");

    // nothing Datasource::setUserName(string user){}
    QC_DATASOURCE->addMethod("setUserName", (q_method_n_t)Datasource_setUserName_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "user");

    // int Datasource::transactionTid(){}
    QC_DATASOURCE->addMethod("transactionTid", (q_method_n_t)Datasource_transactionTid, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto Datasource::vexec(string sql, *softlist<auto> vargs){}
    QC_DATASOURCE->addMethod("vexec", (q_method_n_t)Datasource_vexec_VsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto Datasource::vselect(string sql, *softlist<auto> vargs){}
    QC_DATASOURCE->addMethod("vselect", (q_method_n_t)Datasource_vselect_VsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // *hash<auto> Datasource::vselectRow(string sql, *softlist<auto> vargs){}
    QC_DATASOURCE->addMethod("vselectRow", (q_method_n_t)Datasource_vselectRow_VsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");

    // auto Datasource::vselectRows(string sql, *softlist<auto> vargs){}
    QC_DATASOURCE->addMethod("vselectRows", (q_method_n_t)Datasource_vselectRows_VsC15_softlist_auto_, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "sql", softAutoListOrNothingTypeInfo, QORE_PARAM_NO_ARG, "vargs");
# 177 "QC_Datasource.qpp"
# 184 "QC_Datasource.qpp"
# 187 "QC_Datasource.qpp"
# 190 "QC_Datasource.qpp"
# 193 "QC_Datasource.qpp"
# 196 "QC_Datasource.qpp"
# 199 "QC_Datasource.qpp"
# 202 "QC_Datasource.qpp"
    ns.addConstant("DSDB2", new QoreStringNode("db2"));
    ns.addConstant("DSFreeTDS", new QoreStringNode("freetds"));
    ns.addConstant("DSJDBC", new QoreStringNode("jdbc"));
    ns.addConstant("DSMSSQL", new QoreStringNode("freetds"));
    ns.addConstant("DSMySQL", new QoreStringNode("mysql"));
    ns.addConstant("DSOracle", new QoreStringNode("oracle"));
    ns.addConstant("DSPGSQL", new QoreStringNode("pgsql"));
    ns.addConstant("DSSQLite3", new QoreStringNode("sqlite3"));
    ns.addConstant("DSSybase", new QoreStringNode("sybase"));
# 209 "QC_Datasource.qpp"
# 217 "QC_Datasource.qpp"
# 220 "QC_Datasource.qpp"
# 223 "QC_Datasource.qpp"
# 226 "QC_Datasource.qpp"
# 229 "QC_Datasource.qpp"
# 232 "QC_Datasource.qpp"
# 235 "QC_Datasource.qpp"
# 238 "QC_Datasource.qpp"
# 241 "QC_Datasource.qpp"
# 244 "QC_Datasource.qpp"
# 247 "QC_Datasource.qpp"
# 250 "QC_Datasource.qpp"
# 253 "QC_Datasource.qpp"
# 256 "QC_Datasource.qpp"
# 259 "QC_Datasource.qpp"
# 262 "QC_Datasource.qpp"
    ns.addConstant("DBI_CAP_AUTORECONNECT", ((int64)DBI_CAP_AUTORECONNECT));
    ns.addConstant("DBI_CAP_BIND_BY_PLACEHOLDER", ((int64)DBI_CAP_BIND_BY_PLACEHOLDER));
    ns.addConstant("DBI_CAP_BIND_BY_VALUE", ((int64)DBI_CAP_BIND_BY_VALUE));
    ns.addConstant("DBI_CAP_CHARSET_SUPPORT", ((int64)DBI_CAP_CHARSET_SUPPORT));
    ns.addConstant("DBI_CAP_EVENTS", ((int64)DBI_CAP_EVENTS));
    ns.addConstant("DBI_CAP_HAS_ARRAY_BIND", ((int64)DBI_CAP_HAS_ARRAY_BIND));
    ns.addConstant("DBI_CAP_HAS_DESCRIBE", ((int64)DBI_CAP_HAS_DESCRIBE));
    ns.addConstant("DBI_CAP_HAS_EXECRAW", ((int64)DBI_CAP_HAS_EXECRAW));
    ns.addConstant("DBI_CAP_HAS_NUMBER_SUPPORT", ((int64)DBI_CAP_HAS_NUMBER_SUPPORT));
    ns.addConstant("DBI_CAP_HAS_OPTION_SUPPORT", ((int64)DBI_CAP_HAS_OPTION_SUPPORT));
    ns.addConstant("DBI_CAP_HAS_RESULTSET_OUTPUT", ((int64)DBI_CAP_HAS_RESULTSET_OUTPUT));
    ns.addConstant("DBI_CAP_HAS_SELECT_ROW", ((int64)DBI_CAP_HAS_SELECT_ROW));
    ns.addConstant("DBI_CAP_HAS_STATEMENT", ((int64)DBI_CAP_HAS_STATEMENT));
    ns.addConstant("DBI_CAP_LOB_SUPPORT", ((int64)DBI_CAP_LOB_SUPPORT));
    ns.addConstant("DBI_CAP_SERVER_TIME_ZONE", ((int64)DBI_CAP_SERVER_TIME_ZONE));
    ns.addConstant("DBI_CAP_STORED_PROCEDURES", ((int64)DBI_CAP_STORED_PROCEDURES));
    ns.addConstant("DBI_CAP_TIME_ZONE_SUPPORT", ((int64)DBI_CAP_TIME_ZONE_SUPPORT));
    ns.addConstant("DBI_CAP_TRANSACTION_MANAGEMENT", ((int64)DBI_CAP_TRANSACTION_MANAGEMENT));

    return QC_DATASOURCE;
}
