/* -*- mode: c++; indent-tabs-mode: nil -*- */
/*
    ql_crypto.qpp

    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore/Qore.h>
#include "qore/intern/ql_crypto.h"
#include "qore/intern/EncryptionTransforms.h"

#include <cstdio>
#include <cstdlib>

#include <openssl/err.h>
#include <openssl/rand.h>

#define QCRYPTO_DECRYPT 0
#define QCRYPTO_ENCRYPT 1

extern QoreClass* QC_TRANSFORM;

// maximum length of a MAC value in bytes
#define Q_TAG_MAX 4096

// size of buffer when en/decrypting monolithic data
#define Q_CRYPTO_BUFSIZE 16384

// NOTE: the trailing null ('\0') is not included when encrypting strings or calculating digests

// default initialization vector
static unsigned char def_iv[] = { 0, 0, 0, 0, 0, 0, 0, 0 };

/*
*/
class CryptoHelper : public BaseHelper {
private:
public:
    unsigned char *key[3];
    int keylen[3];

    DLLLOCAL CryptoHelper() : iv(0), output(0), output_len(0), qe(QCS_DEFAULT) {
    }

    DLLLOCAL ~CryptoHelper() {
        if (output)
            free(output);
    }

    DLLLOCAL BinaryNode *getBinary() {
        BinaryNode *b = new BinaryNode(output, output_len);
        output = 0;
        return b;
    }

    DLLLOCAL QoreStringNode *getString(const QoreEncoding *enc = QCS_DEFAULT) {
        // create the string
        QoreStringNode *str = new QoreStringNode((char *)output, output_len, output_len, enc);
        // add trailing '\0'
        str->terminate(output_len);

        output = 0;
        return str;
    }

    DLLLOCAL int setSingleKey(const char *err, const QoreListNode *params, ExceptionSink *xsink, bool has_encoding = false) {
        if (setIV(err, get_param_value(params, 2), xsink))
            return -1;

        setInput(get_param_value(params, 0));
        setKey(params, 1);
        if (has_encoding)
            setEncoding(params, 2);
        return 0;
    }

    DLLLOCAL int doCipher(const EVP_CIPHER* type, const char* cipher, int do_crypt, ExceptionSink *xsink) {
        const char *err = (do_crypt ? "ENCRYPT-ERROR" : "DECRYPT-ERROR");

        QoreEvpCipherCtxHelper ctx;
        EVP_CipherInit_ex(*ctx, type, 0, 0, 0, do_crypt);
        if (key[0]) {
            if (keylen[0] > EVP_MAX_KEY_LENGTH) {
                keylen[0] = EVP_MAX_KEY_LENGTH;
            }

            if (!EVP_CIPHER_CTX_set_key_length(*ctx, keylen[0]) || !EVP_CipherInit_ex(*ctx, 0, 0, key[0], iv, -1)) {
                xsink->raiseException(err, "error setting %s key length=%d", cipher, keylen[0]);
                return -1;
            }
        }

        // we allocate 1 byte more than we need in case we return as a string so we can terminate it
        output = (unsigned char *)malloc(sizeof(char) * (input_len + (EVP_MAX_BLOCK_LENGTH * 2)));

        if (!EVP_CipherUpdate(*ctx, output, &output_len, input, input_len)) {
            xsink->raiseException(err, "error %scrypting %s block", do_crypt ? "en" : "de", cipher);
            return -1;
        }

        int tmplen;
        // Buffer passed to EVP_EncryptFinal() must be after data just encrypted to avoid overwriting it.
        if (!EVP_CipherFinal_ex(*ctx, output + output_len, &tmplen)) {
            xsink->raiseException(err, "error %scrypting final %s block", do_crypt ? "en" : "de", cipher);
            return -1;
        }

        output_len += tmplen;
        //printd(5, "cipher_intern() %s: in=%p (%d) out=%p (%d)\n", cipher, buf, len, cbuf, *size);
        return 0;
    }

    DLLLOCAL int checkKeyLen(const char *err, int n, int len, ExceptionSink *xsink) {
        if (keylen[n] < len) {
            xsink->raiseException(err, "key length is not %d bytes long (%d bytes)", len, keylen[n]);
            return -1;
        }
        keylen[n] = len;
        return 0;
    }

/*
    DLLLOCAL int setDESKey(int n, ExceptionSink *xsink) {
        // force odd parity
        //DES_set_odd_parity((DES_cblock *)key[n]);
        // populate the schedule structure
        //DES_set_key_unchecked((DES_cblock *)key[n], schedule);
        //key[n] = (unsigned char *)schedule;
        return 0;
    }
*/

private:
    unsigned char *iv, *output;
    int output_len;
    const QoreEncoding* qe;

    DLLLOCAL const char* getOrdinal(int n) {
        assert(n > 0 && n <= 3);
        return n == 1 ? "first" : (n == 2 ? "second" : "third");
    }

    DLLLOCAL void setKey(const QoreListNode* args, int n) {
        assert(n > 0 && n <= 3);
        const QoreValue pt = get_param_value(args, n);
        if (pt.getType() == NT_STRING) {
            const QoreStringNode* str = pt.get<const QoreStringNode>();
            key[n - 1] = (unsigned char*)str->getBuffer();
            keylen[n - 1] = str->strlen();
            return;
        }

        const BinaryNode* b = pt.get<const BinaryNode>();
        key[n - 1] = (unsigned char*)b->getPtr();
        keylen[n - 1] = b->size();
    }

    DLLLOCAL void setEncoding(const QoreListNode* args, int n) {
        QoreValue qv = get_param_value(args, n);
        qe = qv.getType() == NT_STRING ? QEM.findCreate(qv.get<const QoreStringNode>()) : QCS_DEFAULT;
    }

    // set the initialization vector (used with hard typing)
    DLLLOCAL int setIV(const char* err, const QoreValue pt, ExceptionSink* xsink) {
        assert(!pt.isNothing());
        if (pt.getType() == NT_STRING) {
            const QoreStringNode* str = pt.get<const QoreStringNode>();
            if (str->strlen() < 8) {
                xsink->raiseException(err, "the input vector must be at least 8 bytes long (%d bytes passed)",
                    str->strlen());
                return -1;
            }
            iv = (unsigned char*)str->getBuffer();
            return 0;
        }

        assert(pt.getType() == NT_BINARY);
        const BinaryNode* b = pt.get<const BinaryNode>();
        if (b->size() < 8) {
            xsink->raiseException(err, "the input vector must be at least 8 bytes long (%d bytes passed)",
                b->size());
            return -1;
        }
        iv = (unsigned char*)b->getPtr();
        return 0;
    }
};

static const EVP_MD* q_lookup_digest(const char* cipher) {
#ifdef OPENSSL_3_PLUS
    return EVP_MD_fetch(nullptr, cipher, nullptr);
#else
    // find digest
    digest_map_t::const_iterator i = digest_map.find(cipher);
    return i == digest_map.end() ? nullptr : i->second;
#endif
}

int HMACHelper::doHMAC(const char* err, const char* digest, const char* ptr, size_t len, ExceptionSink* xsink) {
#ifdef OPENSSL_3_PLUS
    EVP_MAC* mac = EVP_MAC_fetch(nullptr, "hmac", nullptr);
    if (!mac) {
        xsink->raiseException(err, "error looking up MAC '%s'", digest);
        return -1;
    }
    EVP_MAC_CTX* ctx = EVP_MAC_CTX_new(mac);
    if (!ctx) {
        xsink->raiseException(err, "error creating MAC context for MAC '%s'", digest);
        return -1;
    }
    OSSL_PARAM params[2];
    params[0] = OSSL_PARAM_construct_utf8_string("digest", (char*)digest, 0);
    params[1] = OSSL_PARAM_construct_end();

    if (!EVP_MAC_init(ctx, (const unsigned char*)ptr, len, params)) {
        xsink->raiseException("DIGEST-ERROR", "error initialing MAC context for MAC '%s'", digest);
        return -1;
    }
    if (!EVP_MAC_update(ctx, input, input_len)
        || !EVP_MAC_final(ctx, md_value, &len, sizeof(md_value))) {
        xsink->raiseException(err, "error calculating MAC '%s'", digest);
        return -1;
    }
    md_len = (unsigned int)len;
#else
    QoreHmacHelper ctx;
    if (!*ctx) {
        xsink->raiseException(err, "error allocating HMAC object");
        return -1;
    }
    const EVP_MD* md = q_lookup_digest(digest);
    if (!md) {
        xsink->raiseException("DIGEST-ERROR", "error looking up digest '%s' for HMAC", digest);
        return -1;
    }
#ifdef HAVE_OPENSSL_HMAC_RV
    int rc = HMAC_Init_ex(*ctx, ptr, len, md, 0);
    if (!rc) {
        xsink->raiseException(err, "error initializing HMAC");
        return -1;
    }
    if (!HMAC_Update(*ctx, input, input_len)
        || !HMAC_Final(*ctx, md_value, &md_len)) {
        xsink->raiseException(err, "error calculating HMAC");
        return -1;
    }
#else
    HMAC_Init_ex(*ctx, ptr, len, md, 0);
    HMAC_Update(*ctx, input, input_len);
    HMAC_Final(*ctx, md_value, &md_len);
#endif
#endif
    return 0;
}

AbstractQoreNode* missing_openssl_feature(const char *f, ExceptionSink *xsink) {
   return xsink->raiseException("MISSING-FEATURE-ERROR", "the openssl library version that qore was compiled with did not support the %s algorithm; for maximum portability, check Option::HAVE_%s before calling this function", f, f);
}

static int check_mac(const ReferenceNode* mac, int64 mac_size, ExceptionSink* xsink) {
    if (!mac_size) {
        return 0;
    }

    if (!mac) {
        xsink->raiseException("MAC-ERROR", "mac_size argument passed but no mac argument passed");
        return -1;
    }

    if (mac_size < 0) {
        xsink->raiseException("MAC-ERROR", "mac_size argument is negative (" QLLD ")", mac_size);
        return -1;
    }

    if (mac_size > Q_TAG_MAX) {
        xsink->raiseException("MAC-ERROR", "mac_size argument is too large (" QLLD "); maximum value is %d", mac_size,
            Q_TAG_MAX);
        return -1;
    }

    return 0;
}

static QoreValue apply_transformation(SimpleRefHolder<Transform>& t, SimpleRefHolder<BinaryNode>& rv, const char* datastr,
        size_t data_len, ExceptionSink* xsink) {
    // preallocate space for the decrypted data
    rv->preallocate(data_len + (EVP_MAX_BLOCK_LENGTH * 2));
    size_t rv_maxsize = rv->size();
    size_t rvsize = 0;
    size_t offset = 0;

    // decrypt data
    while (offset != data_len) {
        // get block size
        size_t bs = data_len - offset;

        // ensure we always have buffer space for the encryption algorithm
        {
            size_t new_max = rvsize + bs + (EVP_MAX_BLOCK_LENGTH * 2);

            // reallocate target buffer if necessary
            if (new_max > rv_maxsize) {
                rv_maxsize = new_max;
                rv->preallocate(rv_maxsize);
            }
        }

        size_t cbs = rv_maxsize - rvsize;
        std::pair<int64, int64> p = t->apply(datastr + offset, bs, (char*)rv->getPtr() + rvsize, cbs, xsink);
        if (*xsink)
            return QoreValue();
        offset += p.first;
        rvsize += p.second;
    }

    // close the stream
    while (true) {
        // ensure we always have buffer space for the encryption algorithm
        {
            size_t new_max = rvsize + (EVP_MAX_BLOCK_LENGTH * 2);

            // reallocate target buffer if necessary
            if (new_max > rv_maxsize) {
                rv_maxsize = new_max;
                rv->preallocate(rv_maxsize);
            }
        }

        size_t cbs = rv_maxsize - rvsize;
        std::pair<int64, int64> p = t->apply(nullptr, 0, (char*)rv->getPtr() + rvsize, cbs, xsink);
        if (*xsink)
            return QoreValue();
        if (!p.second)
            break;
        rvsize += p.second;
    }

    rv->setSize(rvsize);
    return rv.release();
}

static const EVP_MD* q_get_digest(const QoreString& digest, ExceptionSink* xsink) {
    const EVP_MD* md = q_lookup_digest(digest.c_str());
    if (!md) {
        xsink->raiseException("DIGEST-ERROR", "unknown digest '%s'; check DigestMap for supported digests",
            digest.c_str());
    }
    return md;
}
# 698 "ql_crypto.qpp"
# 733 "ql_crypto.qpp"
# 774 "ql_crypto.qpp"
# 809 "ql_crypto.qpp"
# 844 "ql_crypto.qpp"
# 882 "ql_crypto.qpp"
# 919 "ql_crypto.qpp"
# 956 "ql_crypto.qpp"
# 996 "ql_crypto.qpp"
# 1033 "ql_crypto.qpp"
# 1070 "ql_crypto.qpp"
# 1110 "ql_crypto.qpp"
# 1147 "ql_crypto.qpp"
# 1184 "ql_crypto.qpp"
# 1224 "ql_crypto.qpp"
# 1261 "ql_crypto.qpp"
# 1298 "ql_crypto.qpp"
# 1338 "ql_crypto.qpp"
# 1375 "ql_crypto.qpp"
# 1412 "ql_crypto.qpp"
# 1452 "ql_crypto.qpp"
# 1487 "ql_crypto.qpp"
# 1522 "ql_crypto.qpp"
# 1560 "ql_crypto.qpp"
# 1595 "ql_crypto.qpp"
# 1630 "ql_crypto.qpp"
# 1668 "ql_crypto.qpp"
# 1710 "ql_crypto.qpp"
# 1752 "ql_crypto.qpp"
// binary blowfish_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_blowfish_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 833 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("BLOWFISH-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string blowfish_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_blowfish_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 871 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("BLOWFISH-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary blowfish_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_blowfish_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 798 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("BLOWFISH-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_bf_cbc(), "blowfish", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary cast5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_cast5_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1619 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("CAST5-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string cast5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_cast5_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1657 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("CAST5-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary cast5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_cast5_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1584 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("CAST5-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_cast5_cbc(), "CAST5", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary des_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 944 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
        || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
                return QoreValue();
    }

    return ch.getBinary();
}

// string des_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 984 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
        || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
                return QoreValue();
    }

    return ch.getString();
}

// binary des_ede3_decrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede3_decrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1172 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string des_ede3_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_ede3_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1212 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary des_ede3_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede3_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1135 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_des_ede3_cbc(), "DES", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary des_ede_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1058 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
        || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string des_ede_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_des_ede_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1098 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
        || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary des_ede_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_ede_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1021 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 16, xsink)
        || ch.doCipher(EVP_des_ede_cbc(), "DES", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary des_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_des_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 907 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DES-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DES-KEY-ERROR", 0, 8, xsink)
        || ch.doCipher(EVP_des_cbc(), "DES", QCRYPTO_ENCRYPT, xsink)) {
                return QoreValue();
    }

    return ch.getBinary();
}

// binary des_random_key() {}
static QoreValue f_des_random_key(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 741 "ql_crypto.qpp"
#ifdef OPENSSL_3_PLUS
    QoreEvpCipherCtxHelper ctx;
    if (!ctx) {
        xsink->outOfMemory();
        return QoreValue();
    }
    if (!EVP_CipherInit_ex(*ctx, EVP_des_cbc(), nullptr, nullptr, nullptr, 1)) {
        xsink->raiseException("DES-ERROR", "failed to initialize DES encryption context to generate random key");
        return QoreValue();
    }
    int len = EVP_CIPHER_get_key_length(EVP_des_cbc());
    if (len <= 0) {
        xsink->raiseException("DES-ERROR", "OpenSSL return an invalid DES key length value: %d", len);
        return QoreValue();
    }
    unsigned char* key = (unsigned char*)malloc(sizeof(char) * len);
    if (!key) {
        xsink->outOfMemory();
        return QoreValue();
    }
    if (!EVP_CIPHER_CTX_rand_key(*ctx, key)) {
        xsink->raiseException("DES-ERROR", "OpenSSL failed to generate a DES key");
        return QoreValue();
    }
    return new BinaryNode(key, len);
#else
    DES_cblock* db = (DES_cblock*)malloc(sizeof(DES_cblock));
    DES_random_key(db);
    return new BinaryNode(db, sizeof(DES_cblock));
#endif
    return QoreValue();
}

// binary desx_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_desx_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1286 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DESX-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string desx_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_desx_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1326 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DESX-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary desx_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_desx_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1249 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("DESX-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.checkKeyLen("DESX-KEY-ERROR", 0, 24, xsink)
        || ch.doCipher(EVP_desx_cbc(), "DESX", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary rc2_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc2_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1511 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC2-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string rc2_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc2_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1549 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC2-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary rc2_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc2_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1476 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC2-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc2_cbc(), "rc2", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary rc4_decrypt(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc4_decrypt_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1401 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC4-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// string rc4_decrypt_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc4_decrypt_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1441 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC4-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
}

// binary rc4_encrypt(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc4_encrypt_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1364 "ql_crypto.qpp"
    CryptoHelper ch;

    if (ch.setSingleKey("RC4-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc4(), "rc4", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
}

// binary rc5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc5_decrypt_cbc_VyVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1737 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
    CryptoHelper ch;

    if (ch.setSingleKey("RC5-DECRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
#else
    return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

// string rc5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
static QoreValue f_rc5_decrypt_cbc_to_string_VyVDVDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1782 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
    CryptoHelper ch;

    if (ch.setSingleKey("RC5-DECRYPT-PARAM-ERROR", args, xsink, true)
        || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_DECRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getString();
#else
    return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

// binary rc5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
static QoreValue f_rc5_encrypt_cbc_VDVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1695 "ql_crypto.qpp"
#ifndef OPENSSL_NO_RC5
    CryptoHelper ch;

    if (ch.setSingleKey("RC5-ENCRYPT-PARAM-ERROR", args, xsink)
        || ch.doCipher(EVP_rc5_32_12_16_cbc(), "rc5", QCRYPTO_ENCRYPT, xsink)) {
        return QoreValue();
    }

    return ch.getBinary();
#else
    return missing_openssl_feature("RC5", xsink);
#endif
    return QoreValue();
}

# 1798 "ql_crypto.qpp"
# 1818 "ql_crypto.qpp"
# 1851 "ql_crypto.qpp"
# 1877 "ql_crypto.qpp"
# 1907 "ql_crypto.qpp"
# 1939 "ql_crypto.qpp"
# 1967 "ql_crypto.qpp"
# 2002 "ql_crypto.qpp"
# 2037 "ql_crypto.qpp"
# 2072 "ql_crypto.qpp"
# 2107 "ql_crypto.qpp"
# 2139 "ql_crypto.qpp"
# 2171 "ql_crypto.qpp"
# 2204 "ql_crypto.qpp"
# 2230 "ql_crypto.qpp"
# 2263 "ql_crypto.qpp"
# 2289 "ql_crypto.qpp"
# 2315 "ql_crypto.qpp"
# 2347 "ql_crypto.qpp"
# 2373 "ql_crypto.qpp"
# 2406 "ql_crypto.qpp"
# 2439 "ql_crypto.qpp"
# 2472 "ql_crypto.qpp"
# 2505 "ql_crypto.qpp"
# 2537 "ql_crypto.qpp"
# 2569 "ql_crypto.qpp"
# 2602 "ql_crypto.qpp"
// string DSS(data data) {}
static QoreValue f_DSS_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2126 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    DigestHelper dh(args);
    if (dh.doDigest(DSS_ERR, EVP_dss(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string DSS1(data data) {}
static QoreValue f_DSS1_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2158 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    DigestHelper dh(args);
    if (dh.doDigest(DSS1_ERR, EVP_dss1(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// binary DSS1_bin(data data) {}
static QoreValue f_DSS1_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2556 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    DigestHelper dh(args);
    if (dh.doDigest(DSS1_ERR, EVP_dss1(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// binary DSS_bin(data data) {}
static QoreValue f_DSS_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2524 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    DigestHelper dh(args);
    if (dh.doDigest(DSS_ERR, EVP_dss(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string MD2(data data) {}
static QoreValue f_MD2_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1838 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_MD2) && !defined(NO_MD2)
    DigestHelper dh(args);
    if (dh.doDigest(MD2_ERR, EVP_md2(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// binary MD2_bin(data data) {}
static QoreValue f_MD2_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2250 "ql_crypto.qpp"
#ifdef OPENSSL_HAVE_MDC2
    DigestHelper dh(args);
    if (dh.doDigest(MD2_ERR, EVP_mdc2(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// string MD4(data data) {}
static QoreValue f_MD4_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1868 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(MD4_ERR, EVP_md4(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// binary MD4_bin(data data) {}
static QoreValue f_MD4_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2280 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(MD4_ERR, EVP_md4(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
}

// string MD5(data data) {}
static QoreValue f_MD5_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1898 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(MD5_ERR, EVP_md5(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// binary MD5_bin(data data) {}
static QoreValue f_MD5_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2306 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(MD5_ERR, EVP_md5(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
}

// string MDC2(data data) {}
static QoreValue f_MDC2_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2191 "ql_crypto.qpp"
#ifdef OPENSSL_HAVE_MDC2
    DigestHelper dh(args);
    if (dh.doDigest(MDC2_ERR, EVP_mdc2(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// binary MDC2_bin(data data) {}
static QoreValue f_MDC2_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2589 "ql_crypto.qpp"
#ifdef OPENSSL_HAVE_MDC2
    DigestHelper dh(args);
    if (dh.doDigest(MDC2_ERR, EVP_mdc2(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
   return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// string RIPEMD160(data data) {}
static QoreValue f_RIPEMD160_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2221 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(RIPEMD160_ERR, EVP_ripemd160(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// binary RIPEMD160_binary(data data) {}
static QoreValue f_RIPEMD160_binary_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2619 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(RIPEMD160_ERR, EVP_ripemd160(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
}

// string SHA(data data) {}
static QoreValue f_SHA_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1925 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
    DigestHelper dh(args);
    if (dh.doDigest(SHA_ERR, EVP_sha(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    missing_openssl_feature("SHA", xsink);
    return 0;
#endif
    return QoreValue();
}

// string SHA1(data data) {}
static QoreValue f_SHA1_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1958 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// binary SHA1_bin(data data) {}
static QoreValue f_SHA1_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2364 "ql_crypto.qpp"
    DigestHelper dh(args);
    if (dh.doDigest(SHA1_ERR, EVP_sha1(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
}

// string SHA224(data data) {}
static QoreValue f_SHA224_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 1989 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// binary SHA224_bin(data data) {}
static QoreValue f_SHA224_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2393 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA224_ERR, EVP_sha224(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// string SHA256(data data) {}
static QoreValue f_SHA256_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2024 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// binary SHA256_bin(data data) {}
static QoreValue f_SHA256_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2426 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA256_ERR, EVP_sha256(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// string SHA384(data data) {}
static QoreValue f_SHA384_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2059 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// binary SHA384_bin(data data) {}
static QoreValue f_SHA384_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2459 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA384_ERR, EVP_sha384(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// string SHA512(data data) {}
static QoreValue f_SHA512_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2094 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// binary SHA512_bin(data data) {}
static QoreValue f_SHA512_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2492 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA512) && defined(HAVE_OPENSSL_SHA512)
    DigestHelper dh(args);
    if (dh.doDigest(SHA512_ERR, EVP_sha512(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// binary SHA_bin(data data) {}
static QoreValue f_SHA_bin_VD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 2333 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
    DigestHelper dh(args);
    if (dh.doDigest(SHA_ERR, EVP_sha(), xsink)) {
        return QoreValue();
    }

    return dh.getBinary();
#else
    missing_openssl_feature("SHA", xsink);
    return 0;
#endif
    return QoreValue();
}

# 2630 "ql_crypto.qpp"
# 2691 "ql_crypto.qpp"
# 2720 "ql_crypto.qpp"
# 2751 "ql_crypto.qpp"
# 2788 "ql_crypto.qpp"
# 2819 "ql_crypto.qpp"
# 2856 "ql_crypto.qpp"
# 2893 "ql_crypto.qpp"
# 2930 "ql_crypto.qpp"
# 2967 "ql_crypto.qpp"
# 3003 "ql_crypto.qpp"
# 3039 "ql_crypto.qpp"
# 3073 "ql_crypto.qpp"
# 3102 "ql_crypto.qpp"
# 3136 "ql_crypto.qpp"
# 3173 "ql_crypto.qpp"
# 3200 "ql_crypto.qpp"
# 3253 "ql_crypto.qpp"
# 3306 "ql_crypto.qpp"
# 3372 "ql_crypto.qpp"
# 3433 "ql_crypto.qpp"
# 3503 "ql_crypto.qpp"
# 3546 "ql_crypto.qpp"
# 3581 "ql_crypto.qpp"
# 3604 "ql_crypto.qpp"
// string DSS1_hmac(data data, data key) {}
static QoreValue f_DSS1_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 3022 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC(DSS1_ERR, "dss1", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string DSS_hmac(data data, data key) {}
static QoreValue f_DSS_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2986 "ql_crypto.qpp"
#ifndef HAVE_OPENSSL_INIT_CRYPTO
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC(DSS_ERR, "dss", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    missing_openssl_feature("DSS", xsink);
#endif
    return QoreValue();
}

// string MD2_hmac(data data, data key) {}
static QoreValue f_MD2_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2674 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_MD2) && !defined(NO_MD2)
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("MD2-HMAC-ERROR", "md2", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("MD2", xsink);
#endif
    return QoreValue();
}

// string MD4_hmac(data data, data key) {}
static QoreValue f_MD4_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2708 "ql_crypto.qpp"
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("MD4-HMAC-ERROR", "md4", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// string MD5_hmac(data data, data key) {}
static QoreValue f_MD5_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2739 "ql_crypto.qpp"
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("MD5-HMAC-ERROR", "md5", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// string MDC2_hmac(data data, data key) {}
static QoreValue f_MDC2_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 3056 "ql_crypto.qpp"
#ifdef OPENSSL_HAVE_MDC2
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("MDC2-HMAC-ERROR", "mdc2", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("MDC2", xsink);
#endif
    return QoreValue();
}

// string RIPEMD160_hmac(data data, data key) {}
static QoreValue f_RIPEMD160_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 3090 "ql_crypto.qpp"
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("RIPEMD160-HMAC-ERROR", "ripemd160", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// string SHA1_hmac(data data, data key) {}
static QoreValue f_SHA1_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2807 "ql_crypto.qpp"
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA1-HMAC-ERROR", "sha1", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
}

// string SHA224_hmac(data data, data key) {}
static QoreValue f_SHA224_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2839 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA224-HMAC-ERROR", "sha224", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("SHA224", xsink);
#endif
    return QoreValue();
}

// string SHA256_hmac(data data, data key) {}
static QoreValue f_SHA256_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2876 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA256-HMAC-ERROR", "sha256", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("SHA256", xsink);
#endif
    return QoreValue();
}

// string SHA384_hmac(data data, data key) {}
static QoreValue f_SHA384_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2913 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA384-HMAC-ERROR", "sha384", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("SHA384", xsink);
#endif
    return QoreValue();
}

// string SHA512_hmac(data data, data key) {}
static QoreValue f_SHA512_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2950 "ql_crypto.qpp"
#if !defined(OPENSSL_NO_SHA256) && defined(HAVE_OPENSSL_SHA512)
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA512-HMAC-ERROR", "sha512", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    return missing_openssl_feature("SHA512", xsink);
#endif
    return QoreValue();
}

// string SHA_hmac(data data, data key) {}
static QoreValue f_SHA_hmac_VDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue key = get_param_value(args, 1);
# 2770 "ql_crypto.qpp"
#ifdef HAVE_OPENSSL_SHA
    HMACHelper dh(args);
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);
    if (dh.doHMAC("SHA-HMAC-ERROR", "sha", ptr, len, xsink)) {
        return QoreValue();
    }

    return dh.getString();
#else
    (void)key;
    missing_openssl_feature("SHA", xsink);
    return QoreValue();
#endif
    return QoreValue();
}

// binary decrypt_to_binary(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
static QoreValue f_decrypt_to_binary_VsVDVDVDNDND(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const AbstractQoreNode* mac = get_param_value(args, 4).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
# 3402 "ql_crypto.qpp"
    const char* datastr;
    size_t data_len;
    q_get_data(data, datastr, data_len);

    const char* keystr;
    size_t key_len;
    q_get_data(key, keystr, key_len);

    const char* ivstr;
    size_t iv_len;
    q_get_data(iv, ivstr, iv_len);

    const char* macstr = nullptr;
    size_t mac_len = 0;
    if (mac)
        q_get_data(mac, macstr, mac_len);

    const char* aadstr = nullptr;
    size_t aad_len = 0;
    if (aad)
        q_get_data(aad, aadstr, aad_len);

    SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
    if (*xsink)
        return QoreValue();

    SimpleRefHolder<BinaryNode> rv(new BinaryNode);
    return apply_transformation(t, rv, datastr, data_len, xsink);
}

// string decrypt_to_string(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad, *string encoding) {}
static QoreValue f_decrypt_to_string_VsVDVDVDNDNDNs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const AbstractQoreNode* mac = get_param_value(args, 4).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
    const QoreStringNode* encoding = get_param_value(args, 6).get<const QoreStringNode>();
# 3464 "ql_crypto.qpp"
    const char* datastr;
    size_t data_len;
    q_get_data(data, datastr, data_len);

    const char* keystr;
    size_t key_len;
    q_get_data(key, keystr, key_len);

    const char* ivstr;
    size_t iv_len;
    q_get_data(iv, ivstr, iv_len);

    const char* macstr = nullptr;
    size_t mac_len = 0;
    if (mac)
        q_get_data(mac, macstr, mac_len);

    const char* aadstr = nullptr;
    size_t aad_len = 0;
    if (aad)
        q_get_data(aad, aadstr, aad_len);

    SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
    if (*xsink)
        return QoreValue();

    SimpleRefHolder<BinaryNode> bin(new BinaryNode);
    ValueHolder vh(apply_transformation(t, bin, datastr, data_len, xsink), xsink);
    if (*xsink)
        return QoreValue();

    const QoreEncoding* enc = encoding ? QEM.findCreate(encoding) : QCS_DEFAULT;

    size_t size = vh->get<BinaryNode>()->size();
    // create string with len == allocated to ensure that it gets terminated
    return size ? new QoreStringNode((char*)vh->get<BinaryNode>()->giveBuffer(), size, size, enc) : new QoreStringNode(enc);
}

// binary digest(string digest, data data) {}
static QoreValue f_digest_VsVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* digest = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
# 3123 "ql_crypto.qpp"
    const EVP_MD* md = q_get_digest(digest, xsink);
    if (!md) {
        return QoreValue();
    }

    DigestHelper dh(data);

    return dh.doDigest("DIGEST-CALCULATION-ERROR", md, xsink)
        ? QoreValue()
        : dh.getBinary();
    return QoreValue();
}

// binary encrypt(string alg, data data, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
static QoreValue f_encrypt_VsVDVDVDC18_reference_binary_NiND(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
    QoreValue iv = get_param_value(args, 3);
    const ReferenceNode* mac = get_param_value(args, 4).get<const ReferenceNode>();
    int64 mac_size = HARD_QORE_VALUE_INT(args, 5);
    const AbstractQoreNode* aad = get_param_value(args, 6).get<const AbstractQoreNode>();
# 3338 "ql_crypto.qpp"
    const char* datastr;
    size_t data_len;
    q_get_data(data, datastr, data_len);

    const char* keystr;
    size_t key_len;
    q_get_data(key, keystr, key_len);

    const char* ivstr;
    size_t iv_len;
    q_get_data(iv, ivstr, iv_len);

    if (check_mac(mac, mac_size, xsink)) {
        return QoreValue();
    }

    const char* aadstr = nullptr;
    size_t aad_len = 0;
    if (aad) {
        q_get_data(aad, aadstr, aad_len);
    }

    SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), true, keystr,
            (unsigned)key_len, ivstr, (unsigned)iv_len, nullptr, 0, (unsigned)mac_size, mac, aadstr,
            (unsigned)aad_len, xsink));
    if (*xsink) {
        return QoreValue();
    }

    SimpleRefHolder<BinaryNode> rv(new BinaryNode);
    return apply_transformation(t, rv, datastr, data_len, xsink);
}

// *hash<auto> get_cipher_info(string cipher) {}
static QoreValue f_get_cipher_info_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* cipher = HARD_QORE_VALUE_STRING(args, 0);
# 3560 "ql_crypto.qpp"
    TempEncodingHelper temp(cipher, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

#ifdef OPENSSL_3_PLUS
    const EVP_CIPHER* c = q_lookup_cipher(temp->c_str());
    if (!c) {
        return QoreValue();
    }
    return q_get_cipher_hash(c);
#else
    crypto_map_t::const_iterator i = crypto_map.find(temp->c_str());
    if (i == crypto_map.end()) {
        return QoreValue();
    }
    return i->second.getInfo();
#endif
    return QoreValue();
}

// hash<auto> get_crypto_info() {}
static QoreValue f_get_crypto_info(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3532 "ql_crypto.qpp"
#ifdef OPENSSL_3_PLUS
    return init_cipher_map_hash();
#else
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);

    for (auto& i : crypto_map) {
        rv->setKeyValue(i.first.c_str(), i.second.getInfo(), nullptr);
    }

    return rv.release();
#endif
    return QoreValue();
}

// Transform get_decryptor(string alg, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
static QoreValue f_get_decryptor_VsVDVDNDND(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue key = get_param_value(args, 1);
    QoreValue iv = get_param_value(args, 2);
    const AbstractQoreNode* mac = get_param_value(args, 3).get<const AbstractQoreNode>();
    const AbstractQoreNode* aad = get_param_value(args, 4).get<const AbstractQoreNode>();
# 3283 "ql_crypto.qpp"
    const char* keystr;
    size_t key_len;
    q_get_data(key, keystr, key_len);

    const char* ivstr;
    size_t iv_len;
    q_get_data(iv, ivstr, iv_len);

    const char* macstr = nullptr;
    size_t mac_len = 0;
    if (mac)
        q_get_data(mac, macstr, mac_len);

    const char* aadstr = nullptr;
    size_t aad_len = 0;
    if (aad)
        q_get_data(aad, aadstr, aad_len);

    SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), false, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, macstr, (unsigned)mac_len, 0, nullptr, aadstr, (unsigned)aad_len, xsink));
    return *xsink ? nullptr : new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// list<string> get_digests() {}
static QoreValue f_get_digests(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 3188 "ql_crypto.qpp"
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);

    ReferenceHolder<QoreHashNode> h(init_digest_map_hash(), xsink);
    ConstHashIterator i(*h);
    while (i.next()) {
        rv->push(new QoreStringNode(i.getKey()), xsink);
    }

    return rv.release();
}

// Transform get_encryptor(string alg, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
static QoreValue f_get_encryptor_VsVDVDC18_reference_binary_NiND(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* alg = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue key = get_param_value(args, 1);
    QoreValue iv = get_param_value(args, 2);
    const ReferenceNode* mac = get_param_value(args, 3).get<const ReferenceNode>();
    int64 mac_size = HARD_QORE_VALUE_INT(args, 4);
    const AbstractQoreNode* aad = get_param_value(args, 5).get<const AbstractQoreNode>();
# 3232 "ql_crypto.qpp"
    const char* keystr;
    size_t key_len;
    q_get_data(key, keystr, key_len);

    const char* ivstr;
    size_t iv_len;
    q_get_data(iv, ivstr, iv_len);

    if (check_mac(mac, mac_size, xsink))
        return QoreValue();

    const char* aadstr = nullptr;
    size_t aad_len = 0;
    if (aad)
        q_get_data(aad, aadstr, aad_len);

    SimpleRefHolder<Transform> t(EncryptionTransforms::getCryptoTransform(alg->c_str(), true, keystr, (unsigned)key_len, ivstr, (unsigned)iv_len, nullptr, 0, (unsigned)mac_size, mac, aadstr, (unsigned)aad_len, xsink));
    return *xsink ? nullptr : new QoreObject(QC_TRANSFORM, getProgram(), t.release());
}

// binary get_random_bytes(int size) {}
static QoreValue f_get_random_bytes_Vi(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    int64 size = HARD_QORE_VALUE_INT(args, 0);
# 3620 "ql_crypto.qpp"
    SimpleRefHolder<BinaryNode> rv(new BinaryNode);
    if (size > 0) {
        rv->preallocate(size);
        int rc = RAND_bytes((unsigned char*)rv->getPtr(), size);
        if (rc <= 0) {
            unsigned long err = ERR_get_error();
            char buf[121];
            ERR_error_string(err, buf);
            xsink->raiseException("GET-RANDOM-BYTES-ERROR", "the RAND_bytes() returned error code %d with a request "
                "of length %d: %s", rc, (int)size, buf);
            return QoreValue();
        }
    }
    return rv.release();
}

// bool has_digest(string digest) {}
static QoreValue f_has_digest_Vs(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* digest = HARD_QORE_VALUE_STRING(args, 0);
# 3595 "ql_crypto.qpp"
    TempEncodingHelper temp(digest, QCS_DEFAULT, xsink);
    if (*xsink) {
        return QoreValue();
    }

    return q_lookup_digest(temp->c_str()) ? true : false;
}

// binary hmac(string digest, data data, data key) {}
static QoreValue f_hmac_VsVDVD(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* digest = HARD_QORE_VALUE_STRING(args, 0);
    QoreValue data = get_param_value(args, 1);
    QoreValue key = get_param_value(args, 2);
# 3160 "ql_crypto.qpp"
    HMACHelper dh(data);

    // get key data
    const char* ptr;
    size_t len;
    q_get_data(key, ptr, len);

    return dh.doHMAC("HMAC-CALCULATION-ERROR", digest->c_str(), ptr, len, xsink)
        ? QoreValue()
        : dh.getBinary();
    return QoreValue();
}


DLLLOCAL void init_crypto_functions(QoreNamespace& ns) {

    // initialization code added by qppinit
    SimpleRefHolder<BinaryNode> DefaultIV(new BinaryNode); DefaultIV->append(def_iv, 8);
    // binary blowfish_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("blowfish_decrypt_cbc", (q_func_n_t)f_blowfish_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string blowfish_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("blowfish_decrypt_cbc_to_string", (q_func_n_t)f_blowfish_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary blowfish_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("blowfish_encrypt_cbc", (q_func_n_t)f_blowfish_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary cast5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("cast5_decrypt_cbc", (q_func_n_t)f_cast5_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string cast5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("cast5_decrypt_cbc_to_string", (q_func_n_t)f_cast5_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary cast5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("cast5_encrypt_cbc", (q_func_n_t)f_cast5_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary des_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_decrypt_cbc", (q_func_n_t)f_des_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string des_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_decrypt_cbc_to_string", (q_func_n_t)f_des_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary des_ede3_decrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede3_decrypt_cbc", (q_func_n_t)f_des_ede3_decrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string des_ede3_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_ede3_decrypt_cbc_to_string", (q_func_n_t)f_des_ede3_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary des_ede3_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede3_encrypt_cbc", (q_func_n_t)f_des_ede3_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary des_ede_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede_decrypt_cbc", (q_func_n_t)f_des_ede_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string des_ede_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("des_ede_decrypt_cbc_to_string", (q_func_n_t)f_des_ede_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary des_ede_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_ede_encrypt_cbc", (q_func_n_t)f_des_ede_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary des_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("des_encrypt_cbc", (q_func_n_t)f_des_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary des_random_key() {}
    ns.addBuiltinVariant("des_random_key", (q_func_n_t)f_des_random_key, QCF_CONSTANT, QDOM_DEFAULT, binaryTypeInfo);
    // binary desx_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("desx_decrypt_cbc", (q_func_n_t)f_desx_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string desx_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("desx_decrypt_cbc_to_string", (q_func_n_t)f_desx_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary desx_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("desx_encrypt_cbc", (q_func_n_t)f_desx_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary rc2_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc2_decrypt_cbc", (q_func_n_t)f_rc2_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string rc2_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc2_decrypt_cbc_to_string", (q_func_n_t)f_rc2_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary rc2_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc2_encrypt_cbc", (q_func_n_t)f_rc2_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary rc4_decrypt(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc4_decrypt", (q_func_n_t)f_rc4_decrypt_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string rc4_decrypt_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc4_decrypt_to_string", (q_func_n_t)f_rc4_decrypt_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary rc4_encrypt(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc4_encrypt", (q_func_n_t)f_rc4_encrypt_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // binary rc5_decrypt_cbc(binary data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc5_decrypt_cbc", (q_func_n_t)f_rc5_decrypt_cbc_VyVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string rc5_decrypt_cbc_to_string(binary data, data key, data iv = Qore::DefaultIV, *string encoding) {}
    ns.addBuiltinVariant("rc5_decrypt_cbc_to_string", (q_func_n_t)f_rc5_decrypt_cbc_to_string_VyVDVDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 4, binaryTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary rc5_encrypt_cbc(data data, data key, data iv = Qore::DefaultIV) {}
    ns.addBuiltinVariant("rc5_encrypt_cbc", (q_func_n_t)f_rc5_encrypt_cbc_VDVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv");
    // string DSS(data data) {}
    ns.addBuiltinVariant("DSS", (q_func_n_t)f_DSS_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string DSS1(data data) {}
    ns.addBuiltinVariant("DSS1", (q_func_n_t)f_DSS1_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary DSS1_bin(data data) {}
    ns.addBuiltinVariant("DSS1_bin", (q_func_n_t)f_DSS1_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary DSS_bin(data data) {}
    ns.addBuiltinVariant("DSS_bin", (q_func_n_t)f_DSS_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string MD2(data data) {}
    ns.addBuiltinVariant("MD2", (q_func_n_t)f_MD2_VD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary MD2_bin(data data) {}
    ns.addBuiltinVariant("MD2_bin", (q_func_n_t)f_MD2_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string MD4(data data) {}
    ns.addBuiltinVariant("MD4", (q_func_n_t)f_MD4_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary MD4_bin(data data) {}
    ns.addBuiltinVariant("MD4_bin", (q_func_n_t)f_MD4_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string MD5(data data) {}
    ns.addBuiltinVariant("MD5", (q_func_n_t)f_MD5_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary MD5_bin(data data) {}
    ns.addBuiltinVariant("MD5_bin", (q_func_n_t)f_MD5_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string MDC2(data data) {}
    ns.addBuiltinVariant("MDC2", (q_func_n_t)f_MDC2_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary MDC2_bin(data data) {}
    ns.addBuiltinVariant("MDC2_bin", (q_func_n_t)f_MDC2_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string RIPEMD160(data data) {}
    ns.addBuiltinVariant("RIPEMD160", (q_func_n_t)f_RIPEMD160_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary RIPEMD160_binary(data data) {}
    ns.addBuiltinVariant("RIPEMD160_binary", (q_func_n_t)f_RIPEMD160_binary_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA(data data) {}
    ns.addBuiltinVariant("SHA", (q_func_n_t)f_SHA_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA1(data data) {}
    ns.addBuiltinVariant("SHA1", (q_func_n_t)f_SHA1_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA1_bin(data data) {}
    ns.addBuiltinVariant("SHA1_bin", (q_func_n_t)f_SHA1_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA224(data data) {}
    ns.addBuiltinVariant("SHA224", (q_func_n_t)f_SHA224_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA224_bin(data data) {}
    ns.addBuiltinVariant("SHA224_bin", (q_func_n_t)f_SHA224_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA256(data data) {}
    ns.addBuiltinVariant("SHA256", (q_func_n_t)f_SHA256_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA256_bin(data data) {}
    ns.addBuiltinVariant("SHA256_bin", (q_func_n_t)f_SHA256_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA384(data data) {}
    ns.addBuiltinVariant("SHA384", (q_func_n_t)f_SHA384_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA384_bin(data data) {}
    ns.addBuiltinVariant("SHA384_bin", (q_func_n_t)f_SHA384_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string SHA512(data data) {}
    ns.addBuiltinVariant("SHA512", (q_func_n_t)f_SHA512_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA512_bin(data data) {}
    ns.addBuiltinVariant("SHA512_bin", (q_func_n_t)f_SHA512_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary SHA_bin(data data) {}
    ns.addBuiltinVariant("SHA_bin", (q_func_n_t)f_SHA_bin_VD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // string DSS1_hmac(data data, data key) {}
    ns.addBuiltinVariant("DSS1_hmac", (q_func_n_t)f_DSS1_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string DSS_hmac(data data, data key) {}
    ns.addBuiltinVariant("DSS_hmac", (q_func_n_t)f_DSS_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string MD2_hmac(data data, data key) {}
    ns.addBuiltinVariant("MD2_hmac", (q_func_n_t)f_MD2_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string MD4_hmac(data data, data key) {}
    ns.addBuiltinVariant("MD4_hmac", (q_func_n_t)f_MD4_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string MD5_hmac(data data, data key) {}
    ns.addBuiltinVariant("MD5_hmac", (q_func_n_t)f_MD5_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string MDC2_hmac(data data, data key) {}
    ns.addBuiltinVariant("MDC2_hmac", (q_func_n_t)f_MDC2_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string RIPEMD160_hmac(data data, data key) {}
    ns.addBuiltinVariant("RIPEMD160_hmac", (q_func_n_t)f_RIPEMD160_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA1_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA1_hmac", (q_func_n_t)f_SHA1_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA224_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA224_hmac", (q_func_n_t)f_SHA224_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA256_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA256_hmac", (q_func_n_t)f_SHA256_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA384_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA384_hmac", (q_func_n_t)f_SHA384_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA512_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA512_hmac", (q_func_n_t)f_SHA512_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // string SHA_hmac(data data, data key) {}
    ns.addBuiltinVariant("SHA_hmac", (q_func_n_t)f_SHA_hmac_VDVD, QCF_NO_FLAGS, QDOM_DEFAULT, stringTypeInfo, 2, dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");
    // binary decrypt_to_binary(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
    ns.addBuiltinVariant("decrypt_to_binary", (q_func_n_t)f_decrypt_to_binary_VsVDVDVDNDND, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 6, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "mac", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "aad");
    // string decrypt_to_string(string alg, data data, data key, data iv = Qore::DefaultIV, *data mac, *data aad, *string encoding) {}
    ns.addBuiltinVariant("decrypt_to_string", (q_func_n_t)f_decrypt_to_string_VsVDVDVDNDNDNs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo, 7, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "mac", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "aad", stringOrNothingTypeInfo, QORE_PARAM_NO_ARG, "encoding");
    // binary digest(string digest, data data) {}
    ns.addBuiltinVariant("digest", (q_func_n_t)f_digest_VsVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 2, stringTypeInfo, QORE_PARAM_NO_ARG, "digest", dataTypeInfo, QORE_PARAM_NO_ARG, "data");
    // binary encrypt(string alg, data data, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
    ns.addBuiltinVariant("encrypt", (q_func_n_t)f_encrypt_VsVDVDVDC18_reference_binary_NiND, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 7, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", qore_get_complex_reference_or_nothing_type(binaryTypeInfo), QORE_PARAM_NO_ARG, "mac", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "mac_size", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "aad");
    // *hash<auto> get_cipher_info(string cipher) {}
    ns.addBuiltinVariant("get_cipher_info", (q_func_n_t)f_get_cipher_info_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoHashOrNothingTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "cipher");
    // hash<auto> get_crypto_info() {}
    ns.addBuiltinVariant("get_crypto_info", (q_func_n_t)f_get_crypto_info, QCF_CONSTANT, QDOM_DEFAULT, autoHashTypeInfo);
    // Transform get_decryptor(string alg, data key, data iv = Qore::DefaultIV, *data mac, *data aad) {}
    ns.addBuiltinVariant("get_decryptor", (q_func_n_t)f_get_decryptor_VsVDVDNDND, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 5, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "mac", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "aad");
    // list<string> get_digests() {}
    ns.addBuiltinVariant("get_digests", (q_func_n_t)f_get_digests, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));
    // Transform get_encryptor(string alg, data key, data iv = Qore::DefaultIV, *reference<binary> mac, *int mac_size, *data aad) {}
    ns.addBuiltinVariant("get_encryptor", (q_func_n_t)f_get_encryptor_VsVDVDC18_reference_binary_NiND, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, QC_TRANSFORM->getTypeInfo(), 6, stringTypeInfo, QORE_PARAM_NO_ARG, "alg", dataTypeInfo, QORE_PARAM_NO_ARG, "key", dataTypeInfo, QoreSimpleValue().assign(DefaultIV->refSelf()), "iv", qore_get_complex_reference_or_nothing_type(binaryTypeInfo), QORE_PARAM_NO_ARG, "mac", bigIntOrNothingTypeInfo, QORE_PARAM_NO_ARG, "mac_size", dataOrNothingTypeInfo, QORE_PARAM_NO_ARG, "aad");
    // binary get_random_bytes(int size) {}
    ns.addBuiltinVariant("get_random_bytes", (q_func_n_t)f_get_random_bytes_Vi, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 1, bigIntTypeInfo, QORE_PARAM_NO_ARG, "size");
    // bool has_digest(string digest) {}
    ns.addBuiltinVariant("has_digest", (q_func_n_t)f_has_digest_Vs, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "digest");
    // binary hmac(string digest, data data, data key) {}
    ns.addBuiltinVariant("hmac", (q_func_n_t)f_hmac_VsVDVD, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, binaryTypeInfo, 3, stringTypeInfo, QORE_PARAM_NO_ARG, "digest", dataTypeInfo, QORE_PARAM_NO_ARG, "data", dataTypeInfo, QORE_PARAM_NO_ARG, "key");

}

DLLLOCAL void init_crypto_constants(QoreNamespace& ns) {
# 376 "ql_crypto.qpp"
# 403 "ql_crypto.qpp"
# 408 "ql_crypto.qpp"
# 413 "ql_crypto.qpp"
# 420 "ql_crypto.qpp"
# 427 "ql_crypto.qpp"
# 434 "ql_crypto.qpp"
# 441 "ql_crypto.qpp"
# 448 "ql_crypto.qpp"
# 455 "ql_crypto.qpp"
# 462 "ql_crypto.qpp"
# 469 "ql_crypto.qpp"
# 476 "ql_crypto.qpp"
# 483 "ql_crypto.qpp"
# 489 "ql_crypto.qpp"
# 495 "ql_crypto.qpp"
# 501 "ql_crypto.qpp"
# 508 "ql_crypto.qpp"
# 518 "ql_crypto.qpp"
# 528 "ql_crypto.qpp"
# 538 "ql_crypto.qpp"
# 543 "ql_crypto.qpp"
# 548 "ql_crypto.qpp"
# 553 "ql_crypto.qpp"
# 563 "ql_crypto.qpp"
    ns.addConstant("CRYPTO_ALG_AES_128", new QoreStringNode("aes128"));
    ns.addConstant("CRYPTO_ALG_AES_192", new QoreStringNode("aes192"));
    ns.addConstant("CRYPTO_ALG_AES_256", new QoreStringNode("aes256"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH", new QoreStringNode("blowfish"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH_CFB", new QoreStringNode("blowfish-cfb"));
    ns.addConstant("CRYPTO_ALG_BLOWFISH_OFB", new QoreStringNode("blowfish-ofb"));
    ns.addConstant("CRYPTO_ALG_CAST5", new QoreStringNode("cast5"));
    ns.addConstant("CRYPTO_ALG_CAST5_CFB", new QoreStringNode("cast5-cfb"));
    ns.addConstant("CRYPTO_ALG_CAST5_OFB", new QoreStringNode("cast5-ofb"));
    ns.addConstant("CRYPTO_ALG_DES", new QoreStringNode("des"));
    ns.addConstant("CRYPTO_ALG_DESX", new QoreStringNode("desx"));
    ns.addConstant("CRYPTO_ALG_DES_CFB", new QoreStringNode("des-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE", new QoreStringNode("desede"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3", new QoreStringNode("desede3"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3_CFB", new QoreStringNode("desede3-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE3_OFB", new QoreStringNode("desede3-ofb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE_CFB", new QoreStringNode("desede-cfb"));
    ns.addConstant("CRYPTO_ALG_DES_EDE_OFB", new QoreStringNode("desede-ofb"));
    ns.addConstant("CRYPTO_ALG_DES_OFB", new QoreStringNode("des-ofb"));
    ns.addConstant("CRYPTO_ALG_RC2", new QoreStringNode("rc2"));
    ns.addConstant("CRYPTO_ALG_RC2_CFB", new QoreStringNode("rc2-cfb"));
    ns.addConstant("CRYPTO_ALG_RC2_OFB", new QoreStringNode("rc2-ofb"));
    ns.addConstant("CRYPTO_ALG_RC4", new QoreStringNode("rc4"));
    ns.addConstant("CRYPTO_ALG_RC5", new QoreStringNode("rc5"));
    ns.addConstant("CRYPTO_ALG_RC5_CFB", new QoreStringNode("rc5-cfb"));
    ns.addConstant("CRYPTO_ALG_RC5_OFB", new QoreStringNode("rc5-ofb"));
    BinaryNode* bin_DefaultIV = new BinaryNode;
    bin_DefaultIV->append("\x00\x00\x00\x00\x00\x00\x00\x00", 8);

    ns.addConstant("DefaultIV", bin_DefaultIV);
# 597 "ql_crypto.qpp"
# 620 "ql_crypto.qpp"
# 625 "ql_crypto.qpp"
# 630 "ql_crypto.qpp"
# 635 "ql_crypto.qpp"
# 640 "ql_crypto.qpp"
# 645 "ql_crypto.qpp"
# 650 "ql_crypto.qpp"
# 655 "ql_crypto.qpp"
# 660 "ql_crypto.qpp"
# 665 "ql_crypto.qpp"
# 670 "ql_crypto.qpp"
# 675 "ql_crypto.qpp"
# 680 "ql_crypto.qpp"
    ns.addConstant("CRYPTO_DIGEST_DSS", new QoreStringNode("dss"));
    ns.addConstant("CRYPTO_DIGEST_DSS1", new QoreStringNode("dss1"));
    ns.addConstant("CRYPTO_DIGEST_MD2", new QoreStringNode("md2"));
    ns.addConstant("CRYPTO_DIGEST_MD4", new QoreStringNode("md4"));
    ns.addConstant("CRYPTO_DIGEST_MD5", new QoreStringNode("md5"));
    ns.addConstant("CRYPTO_DIGEST_MDC2", new QoreStringNode("mdc2"));
    ns.addConstant("CRYPTO_DIGEST_RIPEMD160", new QoreStringNode("ripemd160"));
    ns.addConstant("CRYPTO_DIGEST_SHA", new QoreStringNode("sha"));
    ns.addConstant("CRYPTO_DIGEST_SHA1", new QoreStringNode("sha1"));
    ns.addConstant("CRYPTO_DIGEST_SHA224", new QoreStringNode("sha224"));
    ns.addConstant("CRYPTO_DIGEST_SHA256", new QoreStringNode("sha256"));
    ns.addConstant("CRYPTO_DIGEST_SHA384", new QoreStringNode("sha384"));
    ns.addConstant("CRYPTO_DIGEST_SHA512", new QoreStringNode("sha512"));
    ns.addConstant("CipherMap", (init_cipher_map_hash()));
    ns.addConstant("DigestMap", (init_digest_map_hash()));
# 698 "ql_crypto.qpp"
# 733 "ql_crypto.qpp"
# 774 "ql_crypto.qpp"
# 809 "ql_crypto.qpp"
# 844 "ql_crypto.qpp"
# 882 "ql_crypto.qpp"
# 919 "ql_crypto.qpp"
# 956 "ql_crypto.qpp"
# 996 "ql_crypto.qpp"
# 1033 "ql_crypto.qpp"
# 1070 "ql_crypto.qpp"
# 1110 "ql_crypto.qpp"
# 1147 "ql_crypto.qpp"
# 1184 "ql_crypto.qpp"
# 1224 "ql_crypto.qpp"
# 1261 "ql_crypto.qpp"
# 1298 "ql_crypto.qpp"
# 1338 "ql_crypto.qpp"
# 1375 "ql_crypto.qpp"
# 1412 "ql_crypto.qpp"
# 1452 "ql_crypto.qpp"
# 1487 "ql_crypto.qpp"
# 1522 "ql_crypto.qpp"
# 1560 "ql_crypto.qpp"
# 1595 "ql_crypto.qpp"
# 1630 "ql_crypto.qpp"
# 1668 "ql_crypto.qpp"
# 1710 "ql_crypto.qpp"
# 1752 "ql_crypto.qpp"
# 1798 "ql_crypto.qpp"
# 1818 "ql_crypto.qpp"
# 1851 "ql_crypto.qpp"
# 1877 "ql_crypto.qpp"
# 1907 "ql_crypto.qpp"
# 1939 "ql_crypto.qpp"
# 1967 "ql_crypto.qpp"
# 2002 "ql_crypto.qpp"
# 2037 "ql_crypto.qpp"
# 2072 "ql_crypto.qpp"
# 2107 "ql_crypto.qpp"
# 2139 "ql_crypto.qpp"
# 2171 "ql_crypto.qpp"
# 2204 "ql_crypto.qpp"
# 2230 "ql_crypto.qpp"
# 2263 "ql_crypto.qpp"
# 2289 "ql_crypto.qpp"
# 2315 "ql_crypto.qpp"
# 2347 "ql_crypto.qpp"
# 2373 "ql_crypto.qpp"
# 2406 "ql_crypto.qpp"
# 2439 "ql_crypto.qpp"
# 2472 "ql_crypto.qpp"
# 2505 "ql_crypto.qpp"
# 2537 "ql_crypto.qpp"
# 2569 "ql_crypto.qpp"
# 2602 "ql_crypto.qpp"
# 2630 "ql_crypto.qpp"
# 2691 "ql_crypto.qpp"
# 2720 "ql_crypto.qpp"
# 2751 "ql_crypto.qpp"
# 2788 "ql_crypto.qpp"
# 2819 "ql_crypto.qpp"
# 2856 "ql_crypto.qpp"
# 2893 "ql_crypto.qpp"
# 2930 "ql_crypto.qpp"
# 2967 "ql_crypto.qpp"
# 3003 "ql_crypto.qpp"
# 3039 "ql_crypto.qpp"
# 3073 "ql_crypto.qpp"
# 3102 "ql_crypto.qpp"
# 3136 "ql_crypto.qpp"
# 3173 "ql_crypto.qpp"
# 3200 "ql_crypto.qpp"
# 3253 "ql_crypto.qpp"
# 3306 "ql_crypto.qpp"
# 3372 "ql_crypto.qpp"
# 3433 "ql_crypto.qpp"
# 3503 "ql_crypto.qpp"
# 3546 "ql_crypto.qpp"
# 3581 "ql_crypto.qpp"
# 3604 "ql_crypto.qpp"

}
