/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_AbstractClassMember.cpp AbstractClassMember class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_AbstractClassMember.h"
#include "QC_AbstractClass.h"
/* Qore class Qore::Reflection::AbstractClassMember */

qore_classid_t CID_ABSTRACTCLASSMEMBER;
QoreClass* QC_ABSTRACTCLASSMEMBER;

// AbstractClassMember::constructor() {}
static void AbstractClassMember_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 44 "QC_AbstractClassMember.qpp"
xsink->raiseException("ABSTRACTCLASSMEMBER-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// string AbstractClassMember::getAccessModifierString(){}
static QoreValue AbstractClassMember_getAccessModifierString(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 147 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return new QoreStringNode(m->getAccessString());
}

// AbstractClass AbstractClassMember::getClass(){}
static QoreValue AbstractClassMember_getClass(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 250 "QC_AbstractClassMember.qpp"
    return get_class_object(m->pgm, m->cls);
}

// list<string> AbstractClassMember::getModifierList(){}
static QoreValue AbstractClassMember_getModifierList(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 107 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    ReferenceHolder<QoreListNode> rv(new QoreListNode(stringTypeInfo), xsink);
    if (m->is_static) {
        rv->push(new QoreStringNode("static"), xsink);
    }
    else {
        if (static_cast<const QoreExternalNormalMember*>(m->mem)->isTransient()) {
            rv->push(new QoreStringNode("transient"), xsink);
        }
    }

    rv->push(new QoreStringNode(m->getAccessString()), xsink);

    return rv.release();
}

// int AbstractClassMember::getModifiers(){}
static QoreValue AbstractClassMember_getModifiers(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 67 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    int64 rc = 0;
    if (m->is_static) {
        rc |= MC_STATIC;
    }
    else {
        if (static_cast<const QoreExternalNormalMember*>(m->mem)->isTransient()) {
            rc |= MC_TRANSIENT;
        }
    }

    rc |= get_access(m->getAccess());

    return rc;
}

// bool AbstractClassMember::isBuiltin(){}
static QoreValue AbstractClassMember_isBuiltin(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 168 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return m->cls->isSystem();
}

// bool AbstractClassMember::isStatic(){}
static QoreValue AbstractClassMember_isStatic(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 210 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return m->is_static;
}

// bool AbstractClassMember::isTransient(){}
static QoreValue AbstractClassMember_isTransient(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 231 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return !m->is_static && static_cast<const QoreExternalNormalMember*>(m->mem)->isTransient();
}

// bool AbstractClassMember::isUser(){}
static QoreValue AbstractClassMember_isUser(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 189 "QC_AbstractClassMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    return !m->cls->isSystem();
}

DLLLOCAL void preinitAbstractClassMemberClass() {
    QC_ABSTRACTCLASSMEMBER = new QoreClass("AbstractClassMember", "::Qore::Reflection::AbstractClassMember", QDOM_REFLECTION);
    CID_ABSTRACTCLASSMEMBER = QC_ABSTRACTCLASSMEMBER->getID();
    QC_ABSTRACTCLASSMEMBER->setSystem();
}

DLLLOCAL QoreClass* initAbstractClassMemberClass(QoreNamespace& ns) {
    if (!QC_ABSTRACTCLASSMEMBER)
        preinitAbstractClassMemberClass();

    // set parent class
    assert(QC_ABSTRACTMEMBER);
    QC_ABSTRACTCLASSMEMBER->addBuiltinVirtualBaseClass(QC_ABSTRACTMEMBER);

    // AbstractClassMember::constructor() {}
    QC_ABSTRACTCLASSMEMBER->addConstructor(AbstractClassMember_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // string AbstractClassMember::getAccessModifierString(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("getAccessModifierString", (q_method_n_t)AbstractClassMember_getAccessModifierString, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // AbstractClass AbstractClassMember::getClass(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("getClass", (q_method_n_t)AbstractClassMember_getClass, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_ABSTRACTCLASS->getTypeInfo());

    // list<string> AbstractClassMember::getModifierList(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("getModifierList", (q_method_n_t)AbstractClassMember_getModifierList, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, qore_get_complex_list_type(stringTypeInfo));

    // int AbstractClassMember::getModifiers(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("getModifiers", (q_method_n_t)AbstractClassMember_getModifiers, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, bigIntTypeInfo);

    // bool AbstractClassMember::isBuiltin(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("isBuiltin", (q_method_n_t)AbstractClassMember_isBuiltin, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClassMember::isStatic(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("isStatic", (q_method_n_t)AbstractClassMember_isStatic, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClassMember::isTransient(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("isTransient", (q_method_n_t)AbstractClassMember_isTransient, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    // bool AbstractClassMember::isUser(){}
    QC_ABSTRACTCLASSMEMBER->addMethod("isUser", (q_method_n_t)AbstractClassMember_isUser, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, boolTypeInfo);

    return QC_ABSTRACTCLASSMEMBER;
}
