/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_NormalMember.cpp NormalMember class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include <qore_reflection.h>
#include <QC_Class.h>
#include <QC_NormalMember.h>

static int check_member(QoreObject* obj, const char* mname, const QoreClass* cls, ExceptionSink* xsink) {
    if (!obj->validInstanceOf(*cls)) {
        xsink->raiseException("MEMBER-ERROR", "object is of class '%s' which is not compatible with member '%s' of class '%s'", obj->getClassName(), mname, cls->getName());
        return -1;
    }
    return 0;
}
/* Qore class Qore::Reflection::NormalMember */

qore_classid_t CID_NORMALMEMBER;
QoreClass* QC_NORMALMEMBER;

// NormalMember::constructor() {}
static void NormalMember_constructor(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 52 "QC_NormalMember.qpp"
xsink->raiseException("NORMALMEMBER-CONSTRUCTOR-ERROR", "this class cannot be constructed directly or inherited directly by a user-defined class");
}

// auto NormalMember::getValue(object obj){}
static QoreValue NormalMember_getValue_Vo(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
# 69 "QC_NormalMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    // the class ptr must be valid
    if (check_member(obj, m->name.c_str(), m->cls, xsink)) {
        return QoreValue();
    }

    return obj->getReferencedMemberNoMethod(m->name.c_str(), m->cls, xsink);
}

// nothing NormalMember::setValue(object obj, auto val){}
static QoreValue NormalMember_setValue_VoVA(QoreObject* self, QoreReflectionClassMember* m, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreObject* obj = HARD_QORE_VALUE_OBJECT(args, 0);
    QoreValue val = get_param_value(args, 1);
# 97 "QC_NormalMember.qpp"
    // get read access to program object
    QoreExternalProgramContextHelper tch(xsink, m->pgm);
    if (*xsink) {
        return QoreValue();
    }

    if (check_member(obj, m->name.c_str(), m->cls, xsink)) {
        return QoreValue();
    }

    obj->setMemberValue(m->name.c_str(), m->cls, val, xsink);
    return QoreValue();
}

DLLLOCAL void preinitNormalMemberClass() {
    QC_NORMALMEMBER = new QoreClass("NormalMember", "::Qore::Reflection::NormalMember", QDOM_REFLECTION);
    CID_NORMALMEMBER = QC_NORMALMEMBER->getID();
    QC_NORMALMEMBER->setSystem();
}

DLLLOCAL QoreClass* initNormalMemberClass(QoreNamespace& ns) {
    if (!QC_NORMALMEMBER)
        preinitNormalMemberClass();

    // set parent class
    assert(QC_ABSTRACTCLASSMEMBER);
    QC_NORMALMEMBER->addBuiltinVirtualBaseClass(QC_ABSTRACTCLASSMEMBER);

    // NormalMember::constructor() {}
    QC_NORMALMEMBER->addConstructor(NormalMember_constructor, Public, QCF_NO_FLAGS, QDOM_DEFAULT);

    // auto NormalMember::getValue(object obj){}
    QC_NORMALMEMBER->addMethod("getValue", (q_method_n_t)NormalMember_getValue_Vo, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, autoTypeInfo, 1, objectTypeInfo, QORE_PARAM_NO_ARG, "obj");

    // nothing NormalMember::setValue(object obj, auto val){}
    QC_NORMALMEMBER->addMethod("setValue", (q_method_n_t)NormalMember_setValue_VoVA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, nothingTypeInfo, 2, objectTypeInfo, QORE_PARAM_NO_ARG, "obj", autoTypeInfo, QORE_PARAM_NO_ARG, "val");

    return QC_NORMALMEMBER;
}
