/* -*- mode: c++; indent-tabs-mode: nil -*- */
/** @file QC_Type.cpp type class definition */
/*
    Qore Programming Language

    Copyright (C) 2003 - 2023 Qore Technologies, s.r.o.

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Note that the Qore library is released under a choice of three open-source
    licenses: MIT (as above), LGPL 2+, or GPL 2+; see README-LICENSE for more
    information.
*/

#include "qore_reflection.h"
#include "QC_Type.h"
#include "QC_TypedHash.h"
#include "QC_AbstractReflectionFunction.h"

QoreObject* get_type_object(const QoreTypeInfo* t) {
    return new QoreObject(QC_TYPE, getProgram(), new QoreType(t));
}

static QoreHashNode* Type_serializer(const QoreObject& self, const QoreType& t, QoreSerializationContext& context, ExceptionSink* xsink) {
    ReferenceHolder<QoreHashNode> rv(new QoreHashNode(autoTypeInfo), xsink);
    rv->setKeyValue("type", new QoreStringNode(qore_type_get_name(t.typeInfo)), xsink);
    return rv.release();
}

static void Type_deserializer(QoreObject& self, const QoreHashNode* sdata, QoreDeserializationContext& context, ExceptionSink* xsink) {
    if (sdata) {
        QoreValue val = sdata->getKeyValue("type");
        if (val.getType() != NT_STRING) {
            xsink->raiseException("DESERIALIZATION-ERROR", "Type serialization key 'type' has type '%s'; expecting 'string'", val.getTypeName());
            return;
        }
        const QoreTypeInfo* type = qore_get_type_from_string(val.get<const QoreStringNode>()->c_str(), *xsink);
        if (*xsink) {
            return;
        }
        assert(type);
        self.setPrivate(CID_TYPE, new QoreType(type));
        return;
    }

    xsink->raiseException("DESERIALIZATION-ERROR", "missing serialization data for the Type class");
}
/* Qore class Qore::Reflection::Type */

qore_classid_t CID_TYPE;
QoreClass* QC_TYPE;

// auto Type::acceptsValue(auto value){}
static QoreValue Type_acceptsValue_VA(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue value = get_param_value(args, 0);
# 520 "QC_Type.qpp"
    return qore_type_assign_value(t->typeInfo, value, xsink);
}

// bool Type::canConvertToScalar(){}
static QoreValue Type_canConvertToScalar(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 598 "QC_Type.qpp"
    return qore_type_can_convert_to_scalar(t->typeInfo);
}

// Type::constructor(string typestr) {}
static void Type_constructor_Vs(QoreObject* self, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    const QoreStringNode* typestr = HARD_QORE_VALUE_STRING(args, 0);
# 404 "QC_Type.qpp"
TempEncodingHelper utf8_typestr(typestr, QCS_UTF8, xsink);
    const QoreTypeInfo* type = qore_get_type_from_string(utf8_typestr->c_str(), *xsink);
    if (*xsink) {
        return;
    }
    assert(type);
    self->setPrivate(CID_TYPE, new QoreType(type));
}

// hash<string,bool> Type::getAcceptTypeHash(){}
static QoreValue Type_getAcceptTypeHash(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 577 "QC_Type.qpp"
    return qore_type_get_accept_types(t->typeInfo);
}

// Type Type::getBaseType(){}
static QoreValue Type_getBaseType(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 652 "QC_Type.qpp"
    if (!qore_type_is_output_compatible(nothingTypeInfo, t->typeInfo) || t->typeInfo == nothingTypeInfo) {
        return self->refSelf();
    }
    // get type string
    QoreString typestr(typeInfoGetName(t->typeInfo));
    if (typestr[0] != '*') {
        return self->refSelf();
    }
    typestr.splice(0, 1, xsink);
    if (*xsink) {
        return QoreValue();
    }

    const QoreTypeInfo* type = qore_get_type_from_string(typestr.c_str(), *xsink);
    if (*xsink) {
        return QoreValue();
    }
    return new QoreObject(QC_TYPE, getProgram(), new QoreType(type));
}

// int Type::getBaseTypeCode(){}
static QoreValue Type_getBaseTypeCode(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 568 "QC_Type.qpp"
    return qore_type_get_base_type(t->typeInfo);
}

// auto Type::getDefaultValue(){}
static QoreValue Type_getDefaultValue(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 622 "QC_Type.qpp"
    return qore_type_get_default_value(t->typeInfo);
}

// *Type Type::getElementType(){}
static QoreValue Type_getElementType(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 697 "QC_Type.qpp"
    const QoreTypeInfo* element_type = typeInfoGetElementType(t->typeInfo);
    if (!element_type) {
        return QoreValue();
    }
    return new QoreObject(QC_TYPE, getProgram(), new QoreType(element_type));
}

// string Type::getName(){}
static QoreValue Type_getName(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 422 "QC_Type.qpp"
    return new QoreStringNode(qore_type_get_name(t->typeInfo));
}

// Type Type::getOrNothingType(){}
static QoreValue Type_getOrNothingType(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 683 "QC_Type.qpp"
    return new QoreObject(QC_TYPE, getProgram(), new QoreType(get_or_nothing_type_check(t->typeInfo)));
}

// hash<string,bool> Type::getReturnTypeHash(){}
static QoreValue Type_getReturnTypeHash(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 586 "QC_Type.qpp"
    return qore_type_get_return_types(t->typeInfo);
}

// *TypedHash Type::getTypedHash(){}
static QoreValue Type_getTypedHash(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 715 "QC_Type.qpp"
    const TypedHashDecl* hashdecl = typeInfoGetTypedHash(t->typeInfo);
    if (!hashdecl) {
        return QoreValue();
    }
    return new QoreObject(QC_TYPEDHASH, getProgram(), new QoreReflectionTypedHash(getProgram(), hashdecl));
}

// bool Type::hasDefaultValue(){}
static QoreValue Type_hasDefaultValue(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 610 "QC_Type.qpp"
    return qore_type_has_default_value(t->typeInfo);
}

// bool Type::hasType(){}
static QoreValue Type_hasType(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 747 "QC_Type.qpp"
    return typeInfoHasType(t->typeInfo);
}

// bool Type::isAssignableFrom(Type type, reference<bool> may_not_match){}
static QoreValue Type_isAssignableFrom_C4TypeC15reference_bool_(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "Type::isAssignableFrom()", "Type", xsink);
    if (*xsink)
        return 0;
    const ReferenceNode* may_not_match = HARD_QORE_VALUE_REF(args, 1);
# 493 "QC_Type.qpp"
    ReferenceHolder<QoreType> holder(type, xsink);
    QoreTypeSafeReferenceHelper r(may_not_match, xsink);
    if (!r) {
        return QoreValue();
    }

    bool may_not_match_flag = false;
    bool rv = qore_type_is_assignable_from(t->typeInfo, type->typeInfo, may_not_match_flag);
    if (r.assign(may_not_match_flag)) {
        return QoreValue();
    }
    return rv;
}

// bool Type::isAssignableFrom(Type type){}
static QoreValue Type_isAssignableFrom_C4Type(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "Type::isAssignableFrom()", "Type", xsink);
    if (*xsink)
        return 0;
# 537 "QC_Type.qpp"
    ReferenceHolder<QoreType> holder(type, xsink);
    return qore_type_is_assignable_from(t->typeInfo, type->typeInfo);
}

// int Type::isAssignableFrom(auto value){}
static QoreValue Type_isAssignableFrom_VA(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue value = get_param_value(args, 0);
# 554 "QC_Type.qpp"
    return qore_type_is_assignable_from(t->typeInfo, value);
}

// bool Type::isCompatible(Type type){}
static QoreValue Type_isCompatible_C4Type(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "Type::isCompatible()", "Type", xsink);
    if (*xsink)
        return 0;
# 473 "QC_Type.qpp"
    ReferenceHolder<QoreType> holder(type, xsink);
    return qore_type_is_input_output_compatible(t->typeInfo, type->typeInfo);
}

// bool Type::isEqual(Type type){}
static QoreValue Type_isEqual_C4Type(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "Type::isEqual()", "Type", xsink);
    if (*xsink)
        return 0;
# 436 "QC_Type.qpp"
    ReferenceHolder<QoreType> holder(type, xsink);
    return qore_type_equal(t->typeInfo, type->typeInfo);
}

// bool Type::isOrNothingType(){}
static QoreValue Type_isOrNothingType(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 636 "QC_Type.qpp"
    return qore_type_is_output_compatible(nothingTypeInfo, t->typeInfo);
}

// bool Type::isOutputCompatible(Type type){}
static QoreValue Type_isOutputCompatible_C4Type(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    HARD_QORE_VALUE_OBJ_DATA(type, QoreType, args, 0, CID_TYPE, "Type::isOutputCompatible()", "Type", xsink);
    if (*xsink)
        return 0;
# 454 "QC_Type.qpp"
    ReferenceHolder<QoreType> holder(type, xsink);
    return qore_type_is_output_compatible(t->typeInfo, type->typeInfo);
}

// bool Type::isTypedHash(){}
static QoreValue Type_isTypedHash(QoreObject* self, QoreType* t, const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
# 733 "QC_Type.qpp"
    return typeInfoGetTypedHash(t->typeInfo) ? true : false;
}

// static Type Type::getOrNothingType(auto v){}
static QoreValue static_Type_getOrNothingType_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue v = get_param_value(args, 0);
# 773 "QC_Type.qpp"
// the class ptr must be valid
    return new QoreObject(QC_TYPE, getProgram(), new QoreType(get_or_nothing_type_check(v.getFullTypeInfo())));
}

// static Type Type::getType(auto v){}
static QoreValue static_Type_getType_VA(const QoreListNode* args, q_rt_flags_t rtflags, ExceptionSink* xsink) {
    QoreValue v = get_param_value(args, 0);
# 761 "QC_Type.qpp"
return new QoreObject(QC_TYPE, getProgram(), new QoreType(v.getFullTypeInfo()));
}

DLLLOCAL void preinitTypeClass() {
    QC_TYPE = new QoreClass("Type", "::Qore::Reflection::Type", QDOM_REFLECTION);
    CID_TYPE = QC_TYPE->getID();
    QC_TYPE->setSystem();
}

DLLLOCAL QoreClass* initTypeClass(QoreNamespace& ns) {
    if (!QC_TYPE)
        preinitTypeClass();

    // set serializer function
    QC_TYPE->setSerializer(reinterpret_cast<q_serializer_t>(Type_serializer));

    // set deserializer function
    QC_TYPE->setDeserializer(Type_deserializer);

    // set parent class
    assert(QC_SERIALIZABLE);
    QC_TYPE->addBuiltinVirtualBaseClass(QC_SERIALIZABLE);

    // auto Type::acceptsValue(auto value){}
    QC_TYPE->addMethod("acceptsValue", (q_method_n_t)Type_acceptsValue_VA, Public, QCF_NO_FLAGS, QDOM_DEFAULT, autoTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // bool Type::canConvertToScalar(){}
    QC_TYPE->addMethod("canConvertToScalar", (q_method_n_t)Type_canConvertToScalar, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // Type::constructor(string typestr) {}
    QC_TYPE->addConstructor(Type_constructor_Vs, Public, QCF_NO_FLAGS, QDOM_DEFAULT, 1, stringTypeInfo, QORE_PARAM_NO_ARG, "typestr");

    // hash<string,bool> Type::getAcceptTypeHash(){}
    QC_TYPE->addMethod("getAcceptTypeHash", (q_method_n_t)Type_getAcceptTypeHash, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_hash_type(boolTypeInfo));

    // Type Type::getBaseType(){}
    QC_TYPE->addMethod("getBaseType", (q_method_n_t)Type_getBaseType, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // int Type::getBaseTypeCode(){}
    QC_TYPE->addMethod("getBaseTypeCode", (q_method_n_t)Type_getBaseTypeCode, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo);

    // auto Type::getDefaultValue(){}
    QC_TYPE->addMethod("getDefaultValue", (q_method_n_t)Type_getDefaultValue, Public, QCF_CONSTANT, QDOM_DEFAULT, autoTypeInfo);

    // *Type Type::getElementType(){}
    QC_TYPE->addMethod("getElementType", (q_method_n_t)Type_getElementType, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPE->getOrNothingTypeInfo());

    // string Type::getName(){}
    QC_TYPE->addMethod("getName", (q_method_n_t)Type_getName, Public, QCF_RET_VALUE_ONLY, QDOM_DEFAULT, stringTypeInfo);

    // Type Type::getOrNothingType(){}
    QC_TYPE->addMethod("getOrNothingType", (q_method_n_t)Type_getOrNothingType, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPE->getTypeInfo());

    // hash<string,bool> Type::getReturnTypeHash(){}
    QC_TYPE->addMethod("getReturnTypeHash", (q_method_n_t)Type_getReturnTypeHash, Public, QCF_CONSTANT, QDOM_DEFAULT, qore_get_complex_hash_type(boolTypeInfo));

    // *TypedHash Type::getTypedHash(){}
    QC_TYPE->addMethod("getTypedHash", (q_method_n_t)Type_getTypedHash, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPEDHASH->getOrNothingTypeInfo());

    // bool Type::hasDefaultValue(){}
    QC_TYPE->addMethod("hasDefaultValue", (q_method_n_t)Type_hasDefaultValue, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Type::hasType(){}
    QC_TYPE->addMethod("hasType", (q_method_n_t)Type_hasType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Type::isAssignableFrom(Type type, reference<bool> may_not_match){}
    QC_TYPE->addMethod("isAssignableFrom", (q_method_n_t)Type_isAssignableFrom_C4TypeC15reference_bool_, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 2, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type", qore_get_complex_reference_type(boolTypeInfo), QORE_PARAM_NO_ARG, "may_not_match");

    // bool Type::isAssignableFrom(Type type){}
    QC_TYPE->addMethod("isAssignableFrom", (q_method_n_t)Type_isAssignableFrom_C4Type, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // int Type::isAssignableFrom(auto value){}
    QC_TYPE->addMethod("isAssignableFrom", (q_method_n_t)Type_isAssignableFrom_VA, Public, QCF_CONSTANT, QDOM_DEFAULT, bigIntTypeInfo, 1, autoTypeInfo, QORE_PARAM_NO_ARG, "value");

    // bool Type::isCompatible(Type type){}
    QC_TYPE->addMethod("isCompatible", (q_method_n_t)Type_isCompatible_C4Type, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // bool Type::isEqual(Type type){}
    QC_TYPE->addMethod("isEqual", (q_method_n_t)Type_isEqual_C4Type, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // bool Type::isOrNothingType(){}
    QC_TYPE->addMethod("isOrNothingType", (q_method_n_t)Type_isOrNothingType, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // bool Type::isOutputCompatible(Type type){}
    QC_TYPE->addMethod("isOutputCompatible", (q_method_n_t)Type_isOutputCompatible_C4Type, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo, 1, QC_TYPE->getTypeInfo(), QORE_PARAM_NO_ARG, "type");

    // bool Type::isTypedHash(){}
    QC_TYPE->addMethod("isTypedHash", (q_method_n_t)Type_isTypedHash, Public, QCF_CONSTANT, QDOM_DEFAULT, boolTypeInfo);

    // static Type Type::getOrNothingType(auto v){}
    QC_TYPE->addStaticMethod("getOrNothingType", (q_func_n_t)static_Type_getOrNothingType_VA, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPE->getTypeInfo(), 1, autoTypeInfo, QORE_PARAM_NO_ARG, "v");

    // static Type Type::getType(auto v){}
    QC_TYPE->addStaticMethod("getType", (q_func_n_t)static_Type_getType_VA, Public, QCF_CONSTANT, QDOM_DEFAULT, QC_TYPE->getTypeInfo(), 1, autoTypeInfo, QORE_PARAM_NO_ARG, "v");
# 66 "QC_Type.qpp"
# 76 "QC_Type.qpp"
# 81 "QC_Type.qpp"
# 86 "QC_Type.qpp"
# 91 "QC_Type.qpp"
# 96 "QC_Type.qpp"
# 101 "QC_Type.qpp"
# 106 "QC_Type.qpp"
# 111 "QC_Type.qpp"
# 116 "QC_Type.qpp"
# 121 "QC_Type.qpp"
# 126 "QC_Type.qpp"
# 131 "QC_Type.qpp"
# 136 "QC_Type.qpp"
# 141 "QC_Type.qpp"
# 146 "QC_Type.qpp"
# 151 "QC_Type.qpp"
# 156 "QC_Type.qpp"
# 161 "QC_Type.qpp"
# 166 "QC_Type.qpp"
# 171 "QC_Type.qpp"
# 176 "QC_Type.qpp"
# 181 "QC_Type.qpp"
# 186 "QC_Type.qpp"
# 191 "QC_Type.qpp"
# 196 "QC_Type.qpp"
# 201 "QC_Type.qpp"
# 206 "QC_Type.qpp"
# 211 "QC_Type.qpp"
# 216 "QC_Type.qpp"
# 221 "QC_Type.qpp"
# 226 "QC_Type.qpp"
# 231 "QC_Type.qpp"
# 236 "QC_Type.qpp"
# 241 "QC_Type.qpp"
# 246 "QC_Type.qpp"
# 251 "QC_Type.qpp"
# 256 "QC_Type.qpp"
# 261 "QC_Type.qpp"
# 266 "QC_Type.qpp"
# 271 "QC_Type.qpp"
# 276 "QC_Type.qpp"
# 281 "QC_Type.qpp"
# 286 "QC_Type.qpp"
# 291 "QC_Type.qpp"
# 296 "QC_Type.qpp"
# 301 "QC_Type.qpp"
# 306 "QC_Type.qpp"
# 311 "QC_Type.qpp"
# 316 "QC_Type.qpp"
# 321 "QC_Type.qpp"
# 326 "QC_Type.qpp"
# 331 "QC_Type.qpp"
# 336 "QC_Type.qpp"
# 341 "QC_Type.qpp"
# 346 "QC_Type.qpp"
# 351 "QC_Type.qpp"
    ns.addConstant("AutoHashOrNothingType", (get_type_object(autoHashOrNothingTypeInfo)));
    ns.addConstant("AutoHashType", (get_type_object(autoHashTypeInfo)));
    ns.addConstant("AutoListOrNothingType", (get_type_object(autoListOrNothingTypeInfo)));
    ns.addConstant("AutoListType", (get_type_object(autoListTypeInfo)));
    ns.addConstant("AutoType", (get_type_object(autoTypeInfo)));
    ns.addConstant("Base64BinaryOrNothingType", (get_type_object(base64BinaryOrNothingTypeInfo)));
    ns.addConstant("Base64BinaryType", (get_type_object(base64BinaryTypeInfo)));
    ns.addConstant("BinaryOrNothingType", (get_type_object(binaryOrNothingTypeInfo)));
    ns.addConstant("BinaryType", (get_type_object(binaryTypeInfo)));
    ns.addConstant("BoolOrNothingType", (get_type_object(boolOrNothingTypeInfo)));
    ns.addConstant("BoolType", (get_type_object(boolTypeInfo)));
    ns.addConstant("CodeOrNothingType", (get_type_object(codeOrNothingTypeInfo)));
    ns.addConstant("CodeType", (get_type_object(codeTypeInfo)));
    ns.addConstant("DataOrNothingType", (get_type_object(dataOrNothingTypeInfo)));
    ns.addConstant("DataType", (get_type_object(dataTypeInfo)));
    ns.addConstant("DateOrNothingType", (get_type_object(dateOrNothingTypeInfo)));
    ns.addConstant("DateType", (get_type_object(dateTypeInfo)));
    ns.addConstant("FloatOrNothingType", (get_type_object(floatOrNothingTypeInfo)));
    ns.addConstant("FloatType", (get_type_object(floatTypeInfo)));
    ns.addConstant("HashOrNothingType", (get_type_object(hashOrNothingTypeInfo)));
    ns.addConstant("HashType", (get_type_object(hashTypeInfo)));
    ns.addConstant("HexBinaryOrNothingType", (get_type_object(hexBinaryOrNothingTypeInfo)));
    ns.addConstant("HexBinaryType", (get_type_object(hexBinaryTypeInfo)));
    ns.addConstant("IntOrNothingType", (get_type_object(bigIntOrNothingTypeInfo)));
    ns.addConstant("IntType", (get_type_object(bigIntTypeInfo)));
    ns.addConstant("ListOrNothingType", (get_type_object(listOrNothingTypeInfo)));
    ns.addConstant("ListType", (get_type_object(listTypeInfo)));
    ns.addConstant("NothingType", (get_type_object(nothingTypeInfo)));
    ns.addConstant("NullOrNothingType", (get_type_object(nullOrNothingTypeInfo)));
    ns.addConstant("NullType", (get_type_object(nullTypeInfo)));
    ns.addConstant("NumberOrNothingType", (get_type_object(numberOrNothingTypeInfo)));
    ns.addConstant("NumberType", (get_type_object(numberTypeInfo)));
    ns.addConstant("ObjectOrNothingType", (get_type_object(objectOrNothingTypeInfo)));
    ns.addConstant("ObjectType", (get_type_object(objectTypeInfo)));
    ns.addConstant("ReferenceOrNothingType", (get_type_object(referenceOrNothingTypeInfo)));
    ns.addConstant("ReferenceType", (get_type_object(referenceTypeInfo)));
    ns.addConstant("SoftAutoListOrNothingType", (get_type_object(softAutoListOrNothingTypeInfo)));
    ns.addConstant("SoftAutoListType", (get_type_object(softAutoListTypeInfo)));
    ns.addConstant("SoftBinaryOrNothingType", (get_type_object(softBinaryOrNothingTypeInfo)));
    ns.addConstant("SoftBinaryType", (get_type_object(softBinaryTypeInfo)));
    ns.addConstant("SoftBoolOrNothingType", (get_type_object(softBoolOrNothingTypeInfo)));
    ns.addConstant("SoftBoolType", (get_type_object(softBoolTypeInfo)));
    ns.addConstant("SoftDateOrNothingType", (get_type_object(softDateOrNothingTypeInfo)));
    ns.addConstant("SoftDateType", (get_type_object(softDateTypeInfo)));
    ns.addConstant("SoftFloatOrNothingType", (get_type_object(softFloatOrNothingTypeInfo)));
    ns.addConstant("SoftFloatType", (get_type_object(softFloatTypeInfo)));
    ns.addConstant("SoftIntOrNothingType", (get_type_object(softBigIntOrNothingTypeInfo)));
    ns.addConstant("SoftIntType", (get_type_object(softBigIntTypeInfo)));
    ns.addConstant("SoftListOrNothingType", (get_type_object(softListOrNothingTypeInfo)));
    ns.addConstant("SoftListType", (get_type_object(softListTypeInfo)));
    ns.addConstant("SoftNumberOrNothingType", (get_type_object(softNumberOrNothingTypeInfo)));
    ns.addConstant("SoftNumberType", (get_type_object(softNumberTypeInfo)));
    ns.addConstant("SoftStringOrNothingType", (get_type_object(softStringOrNothingTypeInfo)));
    ns.addConstant("SoftStringType", (get_type_object(softStringTypeInfo)));
    ns.addConstant("StringOrNothingType", (get_type_object(stringOrNothingTypeInfo)));
    ns.addConstant("StringType", (get_type_object(stringTypeInfo)));
    ns.addConstant("TimeoutOrNothingType", (get_type_object(timeoutOrNothingTypeInfo)));
    ns.addConstant("TimeoutType", (get_type_object(timeoutTypeInfo)));
# 362 "QC_Type.qpp"
# 372 "QC_Type.qpp"
# 377 "QC_Type.qpp"
# 382 "QC_Type.qpp"
    ns.addConstant("QTI_AMBIGUOUS", ((int64)QTI_AMBIGUOUS));
    ns.addConstant("QTI_IDENT", ((int64)QTI_IDENT));
    ns.addConstant("QTI_NEAR", ((int64)QTI_NEAR));
    ns.addConstant("QTI_NOT_EQUAL", ((int64)QTI_NOT_EQUAL));
    ns.addConstant("QTI_WILDCARD", ((int64)QTI_WILDCARD));

    return QC_TYPE;
}
